﻿' * 名前 : BaseInfoData.vb
' * 機能 : 基本情報ファイルの操作
' * 作成 : 2014/04/-- Ver1.0
' * 更新 : ----/--/-- Ver -.-
' * 備考 : 基本情報ファイルは初期稼働時に作成され、他のファイルの位置やシリアル番号などを保持しています
' *

Imports Microsoft.VisualBasic.ControlChars
Imports System.IO
Imports System.Text
Imports System.Diagnostics

Public Class BaseInfoData
    Public Property DataReady As Integer                               ' 基本情報ファイルが使用可能か否かの状態を表します
    Public Property MyDirectory As String                              ' 基本情報ファイルのディレクトリです
    Public Property MyName As String                                   ' 基本情報ファイルのファイル名です
    Public Property BaseDirectory As String                            ' システムの基準となるディレクトリです
    Public Property CertDirectory As String                            ' 電子証明書を出力するディレクトリです
    Public Property K9piaDirectory As String                           ' 発行局が存在するディレクトリです
    Public Property K9piaName As String                                ' 発行局のファイル名です
    Public Property RegistDirectory As String                          ' 登録情報ファイルを出力するディレクトリです
    Public Property RegistName As String                               ' 登録情報ファイルのファイル名です
    Public Property LogDirectory As String                             ' ログを出力するディレクトリです
    Public Property LogName As String                                  ' ログのファイル名です
    Public Property SerialNumberRoot As Long                           ' 次に作成するルート認証局証明書のシリアル番号です
    Public Property SerialNumberMiddle As Long                         ' 次に作成する中間認証局証明書のシリアル番号です
    Public Property SerialNumberUser As Long                           ' 次に作成する個人証明書のシリアル番号です
    Public Property SerialNumberServer As Long                         ' 次に作成するサーバー証明書のシリアル番号です
    Public Property SerialNumberCrl As Long                            ' 次に作成する失効リストのシリアル番号です
    Public Property CrlCycle As String                                 ' 失効リストの発行周期です

    Public Sub New()                                                   ' コンストラクタ（初期化処理）
        Debug.WriteLine("BaseInfoDataクラスが作成されました。")
        Dim BaseInfoDataFilename As String
        Dim SepaCode(1) As Char
        Me.DataReady = CommonDefines.StatusFileUnknown                ' 変数の初期化
        Me.MyDirectory = "/files/"
        Me.MyName = "BaseInfo"
        Me.BaseDirectory = System.Windows.Forms.Application.StartupPath
        Me.CertDirectory = "/certificate/"
        Me.K9piaDirectory = "/bin/"
        Me.K9piaName = "k9pia.exe"
        Me.RegistDirectory = "/files/"
        Me.RegistName = "Registry"
        Me.LogDirectory = Me.K9piaDirectory & "log/"
        Me.LogName = "k9pca.log"
        Me.SerialNumberRoot = 0
        Me.SerialNumberMiddle = 0
        Me.SerialNumberUser = 0
        Me.SerialNumberServer = 0
        Me.SerialNumberCrl = 0
        Me.CrlCycle = Nothing
        BaseInfoDataFilename = Me.BaseDirectory & Me.MyDirectory & Me.MyName
        SepaCode(0) = ControlChars.Tab
        Try                                                            ' 基本情報ファイルの取り出し
            Debug.WriteLine("基本情報ファイルの取り出しを開始します。")
            Dim FileCheck As FileInfo = New FileInfo(BaseInfoDataFilename)
            If FileCheck.Exists Then
                Using DataFile As StreamReader = New StreamReader(BaseInfoDataFilename)
                    Dim LineBuff As String
                    Do
                        LineBuff = DataFile.ReadLine()
                        If LineBuff IsNot Nothing Then                 ' 1行をタブ区切りで分割します
                            Debug.WriteLine("Line buff [" & LineBuff & "]")
                            Dim ColumnBuff() As String = LineBuff.Split(SepaCode)
                            If ColumnBuff(0) = "cd" Then               ' cd:証明書の出力ディレクトリを取り出します
                                Me.CertDirectory = ColumnBuff(1)
                            ElseIf ColumnBuff(0) = "ad" Then           ' ad:発行局が存在するディレクトリを取り出します
                                Me.K9piaDirectory = ColumnBuff(1)
                            ElseIf ColumnBuff(0) = "an" Then           ' an:発行局のファイル名を取り出します
                                Me.K9piaName = ColumnBuff(1)
                            ElseIf ColumnBuff(0) = "rd" Then           ' rd:登録情報ファイルの出力ディレクトリを取り出します
                                Me.RegistDirectory = ColumnBuff(1)
                            ElseIf ColumnBuff(0) = "rn" Then           ' rn:登録情報ファイルのファイル名を取り出します
                                Me.RegistName = ColumnBuff(1)
                            ElseIf ColumnBuff(0) = "ld" Then           ' ld:ログの出力ディレクトリを取り出します
                                Me.LogDirectory = ColumnBuff(1)
                            ElseIf ColumnBuff(0) = "ln" Then           ' ln:ログのファイル名を取り出します
                                Me.LogName = ColumnBuff(1)
                            ElseIf ColumnBuff(0) = "sr" Then           ' sr:ルート認証局証明書のシリアル番号を取り出します
                                Me.SerialNumberRoot = Integer.Parse(ColumnBuff(1))
                            ElseIf ColumnBuff(0) = "sm" Then           ' sm:中間認証局証明書のシリアル番号を取り出します
                                Me.SerialNumberMiddle = Integer.Parse(ColumnBuff(1))
                            ElseIf ColumnBuff(0) = "su" Then           ' su:個人証明書のシリアル番号を取り出します
                                Me.SerialNumberUser = Integer.Parse(ColumnBuff(1))
                            ElseIf ColumnBuff(0) = "ss" Then           ' ss:サーバー証明書のシリアル番号を取り出します
                                Me.SerialNumberServer = Integer.Parse(ColumnBuff(1))
                            ElseIf ColumnBuff(0) = "sl" Then           ' sl:失効リストのシリアル番号を取り出します
                                Me.SerialNumberCrl = Integer.Parse(ColumnBuff(1))
                            ElseIf ColumnBuff(0) = "lc" Then           ' lc:失効リストの発行周期を取り出します
                                Me.CrlCycle = ColumnBuff(1)
                            Else
                                Debug.WriteLine("基本情報ファイルのデータに誤りがあります。")
                                Debug.WriteLine("ID [" & ColumnBuff(0) & "]を無視します。")
                            End If
                        End If
                    Loop Until LineBuff Is Nothing
                    DataFile.Close()
                End Using
                Me.DataReady = CommonDefines.StatusFileReady
                Debug.WriteLine("基本情報ファイルが準備できました。")
            Else
                Me.DataReady = CommonDefines.StatusFileNotFound
                Debug.WriteLine("基本情報ファイルの取り出しに失敗しました。")
                Debug.WriteLine("基本情報ファイルが存在しません。")
            End If
        Catch ex As DirectoryNotFoundException                         ' ディレクトリが存在しない場合
            Me.DataReady = CommonDefines.StatusFileNotDirectory
            Debug.WriteLine("基本情報ファイルの取り出しに失敗しました。")
            Debug.WriteLine(ex.Message)
        Catch ex As FileNotFoundException                              ' ファイルが存在しない場合
            Me.DataReady = CommonDefines.StatusFileNotFound
            Debug.WriteLine("基本情報ファイルの取り出しに失敗しました。")
            Debug.WriteLine(ex.Message)
        Catch ex As Exception                                          ' それ以外の異常が発生した場合
            Me.DataReady = CommonDefines.StatusFileNotReady
            Debug.WriteLine("基本情報ファイルの取り出しに失敗しました。")
            Debug.WriteLine(ex.Message)
        End Try
        Debug.WriteLine("MyDirectory       [" & Me.MyDirectory & "]")
        Debug.WriteLine("MyName            [" & Me.MyName & "]")
        Debug.WriteLine("BaseDirectory     [" & Me.BaseDirectory & "]")
        Debug.WriteLine("CertDirectory     [" & Me.CertDirectory & "]")
        Debug.WriteLine("K9piaDirectory    [" & Me.K9piaDirectory & "]")
        Debug.WriteLine("K9piaName         [" & Me.K9piaName & "]")
        Debug.WriteLine("RegistDirectory   [" & Me.RegistDirectory & "]")
        Debug.WriteLine("RegistName        [" & Me.RegistName & "]")
        Debug.WriteLine("LogDirectory      [" & Me.LogDirectory & "]")
        Debug.WriteLine("LogName           [" & Me.LogName & "]")
    End Sub

    Public Sub WriteFile()                                             ' 基本情報ファイルの書き込み
        Dim BaseInfoDataFilename As String = Me.BaseDirectory & Me.MyDirectory & Me.MyName
        Debug.WriteLine("基本情報ファイルの書き込みを開始します。")
        If Me.DataReady <> CommonDefines.StatusFileNotReady Then
            Try                                                        ' ディレクトリが存在しない場合には作成します
                Dim CheckDir As DirectoryInfo = New DirectoryInfo(Me.BaseDirectory & Me.MyDirectory)
                If Not CheckDir.Exists Then
                    CheckDir.Create()
                    Debug.WriteLine("ディレクトリ" & Me.MyDirectory & "を作成しました。")
                End If                                                 ' 新規・上書きで基本情報ファイルを作成します
                Using DataFile As StreamWriter = New StreamWriter(BaseInfoDataFilename, False)
                    Dim LineBuff As String
                    LineBuff = String.Format("cd{0}{1}", ControlChars.Tab, Me.CertDirectory)
                    DataFile.WriteLine(LineBuff)                       ' cd:証明書の出力ディレクトリを書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("ad{0}{1}", ControlChars.Tab, Me.K9piaDirectory)
                    DataFile.WriteLine(LineBuff)                       ' ad:発行局が存在するディレクトリを書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("an{0}{1}", ControlChars.Tab, Me.K9piaName)
                    DataFile.WriteLine(LineBuff)                       ' an:発行局のファイル名を書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("rd{0}{1}", ControlChars.Tab, Me.RegistDirectory)
                    DataFile.WriteLine(LineBuff)                       ' rd:登録情報ファイルの出力ディレクトリを書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("rn{0}{1}", ControlChars.Tab, Me.RegistName)
                    DataFile.WriteLine(LineBuff)                       ' rn:登録情報ファイルのファイル名を書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("ld{0}{1}", ControlChars.Tab, Me.LogDirectory)
                    DataFile.WriteLine(LineBuff)                       ' ld:ログの出力ディレクトリを書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("ln{0}{1}", ControlChars.Tab, Me.LogName)
                    DataFile.WriteLine(LineBuff)                       ' ln:ログのファイル名を書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("sr{0}{1}", ControlChars.Tab, Me.SerialNumberRoot)
                    DataFile.WriteLine(LineBuff)                       ' sr:ルート認証局証明書のシリアル番号を書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("sm{0}{1}", ControlChars.Tab, Me.SerialNumberMiddle)
                    DataFile.WriteLine(LineBuff)                       ' sm:中間認証局証明書のシリアル番号を書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("su{0}{1}", ControlChars.Tab, Me.SerialNumberUser)
                    DataFile.WriteLine(LineBuff)                       ' su:個人証明書のシリアル番号を書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("ss{0}{1}", ControlChars.Tab, Me.SerialNumberServer)
                    DataFile.WriteLine(LineBuff)                       ' ss:サーバー証明書のシリアル番号を書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("sl{0}{1}", ControlChars.Tab, Me.SerialNumberCrl)
                    DataFile.WriteLine(LineBuff)                       ' sl:失効リストのシリアル番号を書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    LineBuff = String.Format("lc{0}{1}", ControlChars.Tab, Me.CrlCycle)
                    DataFile.WriteLine(LineBuff)                       ' lc:失効リストの発行周期を書き込みます
                    Debug.WriteLine("Line buff [" & LineBuff & "]")
                    DataFile.Close()
                End Using
                Debug.WriteLine("基本情報ファイルの書き込みが正常終了しました。")
            Catch ex As Exception
                Debug.WriteLine("基本情報ファイルの書き込みに失敗しました。")
                Debug.WriteLine(ex.Message)
            End Try
        Else
            Debug.WriteLine("エラーのため処理を中止しました。")
        End If
    End Sub
    ' 番号有無のチェックを追加する
    Public Sub NextSerialNumberRoot()                                  ' ルート認証局証明書のシリアル番号をカウントアップします
        Me.SerialNumberRoot += 1
        Debug.WriteLine("ルート認証局証明書のシリアル番号を更新しました。[" & Me.SerialNumberRoot & "]")
    End Sub

    Public Sub NextSerialNumberMiddle()                                ' 中間認証局証明書のシリアル番号をカウントアップします
        Me.SerialNumberMiddle += 1
        Debug.WriteLine("中間認証局証明書のシリアル番号を更新しました。[" & Me.SerialNumberMiddle & "]")
    End Sub

    Public Sub NextSerialNumberUser()                                  ' 個人証明書のシリアル番号をカウントアップします
        Me.SerialNumberUser += 1
        Debug.WriteLine("個人証明書のシリアル番号を更新しました。[" & Me.SerialNumberUser & "]")
    End Sub

    Public Sub NextSerialNumberServer()                                ' サーバー証明書のシリアル番号をカウントアップします
        Me.SerialNumberServer += 1
        Debug.WriteLine("サーバー証明書のシリアル番号を更新しました。[" & Me.SerialNumberServer & "]")
    End Sub

    Public Sub NextSerialNumberCrl()                                   ' 失効リストのシリアル番号をカウントアップします
        Me.SerialNumberCrl += 1
        Debug.WriteLine("失効リストのシリアル番号を更新しました。[" & Me.SerialNumberCrl & "]")
    End Sub
End Class
