﻿' * 名前 : CsrAnalyze.vb
' * 機能 : CSRファイルの読み取りと解析
' * 作成 : 2014/--/-- Ver1.0
' * 更新 : ----/--/-- Ver -.-
' * 備考 : 
' *

Imports Microsoft.VisualBasic.ControlChars
Imports System.IO
Imports System.Text
Imports System.Diagnostics
Imports System.Security.Cryptography

Public Class CsrAnalyze
    Public Property ReturnCode As Integer
    Public Property ErrorMessage As String
    Public Property Subject As String
    Public Property ObjectID As ArrayList
    Public Property ValueText As ArrayList

    Const ASN1_BOL As Byte = &H1
    Const ASN1_INT As Byte = &H2
    Const ASN1_BIT As Byte = &H3
    Const ASN1_OCT As Byte = &H4
    Const ASN1_NUL As Byte = &H5
    Const ASN1_OID As Byte = &H6
    Const ASN1_ENM As Byte = &HA
    Const ASN1_UTF As Byte = &HC
    Const ASN1_PRT As Byte = &H13
    Const ASN1_TEL As Byte = &H14
    Const ASN1_VDO As Byte = &H15
    Const ASN1_IA5 As Byte = &H16
    Const ASN1_BMP As Byte = &H1E
    Const ASN1_UTC As Byte = &H17
    Const ASN1_SEQ As Byte = &H30
    Const ASN1_SET As Byte = &H31
    Const ASN1_CHO As Byte = &H80

    Public Sub New()
        Me.ReturnCode = -1
        Me.ErrorMessage = ""
        Me.Subject = ""
        Me.ObjectID = New ArrayList()
        Me.ValueText = New ArrayList()
    End Sub

    Public Sub ReadAndAnalyze(ByVal Filename As String)
        Dim BinaryBuff As Byte() = Nothing
        Dim BinaryBuff1st As Byte
        Dim CheckReader As BinaryReader = New BinaryReader(File.Open(Filename, FileMode.Open))
        BinaryBuff1st = CheckReader.ReadByte()
        CheckReader.Close()
        If BinaryBuff1st = &H30 Then                                   ' CSRをバイナリデータとして読み込みます
            Try
                Dim CsrReader As BinaryReader = New BinaryReader(File.Open(Filename, FileMode.Open))
                BinaryBuff = CsrReader.ReadBytes(4096)
                CsrReader.Close()
            Catch ex As Exception
                Me.ErrorMessage = "CSRファイルの読み込みに失敗しました。"
                Debug.WriteLine(Me.ErrorMessage)
                Me.ReturnCode = -3
            End Try
        ElseIf BinaryBuff1st = &H2D Then                               ' CSRをBase64テキストとして読み込みます
            Dim LineBuff() As String = {0}
            Dim LineId As Integer = 0
            Try
                Dim CsrReader As StreamReader = New StreamReader(Filename)
                Do While CsrReader.Peek() > -1
                    Array.Resize(LineBuff, (LineId + 1))
                    LineBuff(LineId) = CsrReader.ReadLine().Trim()
                    LineId += 1
                Loop
                CsrReader.Close()
                If LineId > 2 Then
                    If (LineBuff(0) = CommonDefines.CSRheader) And (LineBuff(LineId - 1) = CommonDefines.CSRtailer) Then
                        Dim Base64Buff As String = ""
                        For i As Integer = 1 To LineId - 2 Step 1
                            Base64Buff &= LineBuff(i)
                        Next
                        BinaryBuff = Convert.FromBase64String(Base64Buff).Clone()
                    ElseIf (LineBuff(0) = CommonDefines.CSRheader2) And (LineBuff(LineId - 1) = CommonDefines.CSRtailer2) Then
                        Dim Base64Buff As String = ""
                        For i As Integer = 1 To LineId - 2 Step 1
                            Base64Buff &= LineBuff(i)
                        Next
                        BinaryBuff = Convert.FromBase64String(Base64Buff).Clone()
                    Else
                        LineId = 0
                        Me.ErrorMessage = "CSRファイルの構造に間違いがあります。"
                        Debug.WriteLine(Me.ErrorMessage)
                        Me.ReturnCode = -5
                    End If
                Else
                    LineId = 0
                    Me.ErrorMessage = "CSRファイルの読み込みに失敗しました。"
                    Debug.WriteLine(Me.ErrorMessage)
                    Me.ReturnCode = -4
                End If
            Catch ex As Exception
                LineId = 0
                Me.ErrorMessage = "CSRファイルの読み込みに失敗しました。"
                Debug.WriteLine(Me.ErrorMessage)
                Me.ReturnCode = -3
            End Try
        Else
            Me.ErrorMessage = "ファイルはCSRではありません。"
            Debug.WriteLine(Me.ErrorMessage)
            Me.ReturnCode = -2
        End If
        If IsNothing(BinaryBuff) Then
            Me.ErrorMessage = "CSRバイナリデータが存在しません。"
            Debug.WriteLine(Me.ErrorMessage)
        Else                                                           ' 読み込んだCSRを解析します
            Dim Csr1st As ASN1analyze = New ASN1analyze()
            If Csr1st.RunAnalyze(BinaryBuff) Then
                If Me.CheckLegal(Csr1st) Then
                    Dim Csr2nd_base As ASN1analyze = Csr1st.ChildElement(0)
                    Dim Csr3rd_subject As ASN1analyze = Csr2nd_base.ChildElement(1)
                    For i As Integer = 0 To Csr3rd_subject.ChildCount - 1 Step 1
                        Dim Csr4th_set As ASN1analyze = Csr3rd_subject.ChildElement(i)
                        Dim Csr5th_seq As ASN1analyze = Csr4th_set.ChildElement(0)
                        Dim Csr6th_oid As ASN1analyze = Csr5th_seq.ChildElement(0)
                        Dim Csr6th_val As ASN1analyze = Csr5th_seq.ChildElement(1)
                        Dim ElemOid As String = Csr6th_oid.GetOID()
                        If ElemOid = CommonDefines.OIDcountryName Then
                            Me.ObjectID.Add(CommonDefines.OIDcountryName)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", C=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDstateOrProvinceName Then
                            Me.ObjectID.Add(CommonDefines.OIDstateOrProvinceName)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", S=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDlocalityName Then
                            Me.ObjectID.Add(CommonDefines.OIDlocalityName)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", L=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDorganizationName Then
                            Me.ObjectID.Add(CommonDefines.OIDorganizationName)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", O=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDorganizationUnitName Then
                            Me.ObjectID.Add(CommonDefines.OIDorganizationUnitName)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", OU=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDcommonName Then
                            Me.ObjectID.Add(CommonDefines.OIDcommonName)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", CN=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDemail Then
                            Me.ObjectID.Add(CommonDefines.OIDemail)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", Email=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDpostalCode Then
                            Me.ObjectID.Add(CommonDefines.OIDpostalCode)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", PostalCode=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDstreetAddress Then
                            Me.ObjectID.Add(CommonDefines.OIDstreetAddress)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", Street=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDJOIcountryName Then
                            Me.ObjectID.Add(CommonDefines.OIDJOIcountryName)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", JurisdictionOfIncorporationCountryName=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDJOIstateOrProvinceName Then
                            Me.ObjectID.Add(CommonDefines.OIDJOIstateOrProvinceName)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", JurisdictionOfIncorporationStateOrProvinceName=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDJOIlocalityName Then
                            Me.ObjectID.Add(CommonDefines.OIDJOIlocalityName)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", JurisdictionOfIncorporationLocalityName=" & Csr6th_val.GetText()
                        ElseIf ElemOid = CommonDefines.OIDJOInumber Then
                            Me.ObjectID.Add(CommonDefines.OIDJOInumber)
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", JurisdictionOfIncorporationNumber=" & Csr6th_val.GetText()
                        Else
                            Me.ObjectID.Add(Csr6th_oid.GetOID())
                            Me.ValueText.Add(Csr6th_val.GetText())
                            Me.Subject &= ", " & Csr6th_oid.GetOID() & "=" & Csr6th_val.GetText()
                        End If
                    Next
                    Me.Subject = Me.Subject.Substring(2)
                    Me.ReturnCode = 0
                Else
                    ' Me.ErrorMessage = "エラーメッセージはCheckLegalでセットします。"
                    ' Debug.WriteLine(Me.ErrorMessage)
                    Debug.WriteLine("CSRファイルのフォーマットに間違いがあります。")
                    Me.ReturnCode = -7
                End If
            Else
                Me.ErrorMessage = "CSRファイルが解析できません。"
                Debug.WriteLine(Me.ErrorMessage)
                Me.ReturnCode = -6
            End If
        End If
    End Sub

    Function CheckLegal(ByRef Csr1st As ASN1analyze) As Boolean
        Dim RetFlag As Boolean = False
        Do
            If (Csr1st.MyType <> ASN1_SEQ) Or (Csr1st.ChildCount <> 3) Then
                Me.ErrorMessage = "CSRの第1階層に間違いがあります。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If

            Dim Csr2nd_base As ASN1analyze = Csr1st.ChildElement(0)
            Dim Csr2nd_algo As ASN1analyze = Csr1st.ChildElement(1)
            Dim Csr2nd_sign As ASN1analyze = Csr1st.ChildElement(2)

            If (Csr2nd_algo.MyType <> ASN1_SEQ) Or (Csr2nd_algo.ChildCount <> 2) Then
                Me.ErrorMessage = "CSRのアルゴリズムが特定できません。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If

            Dim Csr3rd_oid As ASN1analyze = Csr2nd_algo.ChildElement(0)
            Dim Csr3rd_prm As ASN1analyze = Csr2nd_algo.ChildElement(1)

            If (Csr3rd_oid.GetOID() <> "1.2.840.113549.1.1.5") And (Csr3rd_oid.GetOID() <> "1.2.840.113549.1.1.11") Then
                Me.ErrorMessage = "CSRのアルゴリズムはサポートしていません。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If
            If Csr3rd_prm.MyType <> ASN1_NUL Then
                Me.ErrorMessage = "CSRのアルゴリズムに余分なパラメータがあります。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If

            If (Csr2nd_base.MyType <> ASN1_SEQ) Or (Csr2nd_base.ChildCount < 3) Then
                Me.ErrorMessage = "CSRの基本領域が特定できません。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If

            Dim Csr3rd_ver As ASN1analyze = Csr2nd_base.ChildElement(0)
            Dim Csr3rd_sub As ASN1analyze = Csr2nd_base.ChildElement(1)
            Dim Csr3rd_key As ASN1analyze = Csr2nd_base.ChildElement(2)

            If (Csr3rd_ver.MyType <> ASN1_INT) Or (Csr3rd_ver.GetInt64 <> 0) Then
                Me.ErrorMessage = "CSRのバージョンはサポートしていません。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If

            If (Csr3rd_sub.MyType <> ASN1_SEQ) Or (Csr3rd_sub.ChildCount = 0) Then
                Me.ErrorMessage = "CSRの作成者が特定できません。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If
            For i As Integer = 0 To Csr3rd_sub.ChildCount - 1 Step 1
                Dim Csr4th_set As ASN1analyze = Csr3rd_sub.ChildElement(i)
                If (Csr4th_set.MyType <> ASN1_SET) Or (Csr4th_set.ChildCount <> 1) Then
                    Me.ErrorMessage = "CSRの作成者に間違いがあります。"
                    Debug.WriteLine(Me.ErrorMessage)
                    Exit Do
                End If
                Dim Csr5th_seq As ASN1analyze = Csr4th_set.ChildElement(0)
                If (Csr5th_seq.MyType <> ASN1_SEQ) Or (Csr5th_seq.ChildCount <> 2) Then
                    Me.ErrorMessage = "CSRの作成者が間違ってます。"
                    Debug.WriteLine(Me.ErrorMessage)
                    Exit Do
                End If
                Dim Csr6th_oid As ASN1analyze = Csr5th_seq.ChildElement(0)
                Dim Csr6th_val As ASN1analyze = Csr5th_seq.ChildElement(1)
                If Csr6th_oid.MyType <> ASN1_OID Then
                    Me.ErrorMessage = "CSRの作成者が理解できません。"
                    Debug.WriteLine(Me.ErrorMessage)
                    Exit Do
                End If
            Next

            If (Csr3rd_key.MyType <> ASN1_SEQ) Or (Csr3rd_key.ChildCount <> 2) Then
                Me.ErrorMessage = "CSRの公開鍵が特定できません。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If
            Dim Csr4th_seq As ASN1analyze = Csr3rd_key.ChildElement(0)
            Dim Csr4th_bit As ASN1analyze = Csr3rd_key.ChildElement(1)
            If (Csr4th_seq.MyType <> ASN1_SEQ) Or (Csr4th_seq.ChildCount <> 2) Then
                Me.ErrorMessage = "CSRの公開鍵に間違いがあります。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If
            If (Csr4th_bit.MyType <> ASN1_BIT) Or (Csr4th_bit.ChildCount <> 1) Then
                Me.ErrorMessage = "CSRの公開鍵がBitではありません。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If
            Dim Csr5th_bit As ASN1analyze = Csr4th_bit.ChildElement(0)
            If (Csr5th_bit.MyType <> ASN1_SEQ) Or (Csr5th_bit.ChildCount <> 2) Then
                Me.ErrorMessage = "CSRの公開鍵が間違ってます。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If
            Dim Csr6th_mod As ASN1analyze = Csr5th_bit.ChildElement(0)
            Dim Csr6th_exp As ASN1analyze = Csr5th_bit.ChildElement(1)
            If (Csr6th_mod.MyType <> ASN1_INT) Or (Csr6th_mod.MyData(0) <> &H0) Then
                Me.ErrorMessage = "CSRの公開鍵のModulusが間違っています。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If
            If (Csr6th_exp.MyType <> ASN1_INT) Or (Csr6th_exp.GetInt64 <> 65537) Then
                Me.ErrorMessage = "CSRの公開鍵のPublicExponentが間違っています。"
                Debug.WriteLine(Me.ErrorMessage)
                Exit Do
            End If
            ' 本当は署名の検証するべきなんだけど、面倒なんでパス！
            ' CSRを偽造したところで秘密鍵が無ければ証明書は役に立たない。
            ' やってみたい人は自分で調べてね。
            ' キーワードは「CERT_REQUEST_INFO structure」です。
            RetFlag = True
        Loop While False
        Return RetFlag
    End Function
End Class
