﻿' * 名前 : RegistData.vb
' * 機能 : 登録ファイルの操作
' * 作成 : 2014/04/-- Ver1.0
' * 更新 : ----/--/-- Ver -.-
' * 備考 : 登録情報ファイルは初期稼働時に作成され、認証局やユーサー・サーバーの証明書内容を保持しています
' *

Imports Microsoft.VisualBasic.ControlChars
Imports System.IO
Imports System.Text
Imports System.Diagnostics
Imports System.Collections

Public Class RegistData
    Public Property DataReady As Integer                               ' 登録情報ファイルが使用可能か否かの状態を表します
    Public Property RegistDirectory As String                          ' 登録情報ファイルのディレクトリです
    Public Property RegistName As String                               ' 登録情報ファイルのファイル名です

    Private Buffering As Boolean                                       ' 登録情報ファイルのデータがあるか否かを保持します
    Private BuffSize As Integer                                        ' 登録情報ファイルのデータ数を保持します
    Private NumBuff As ArrayList                                       ' Number データ(第一項目：ブロック番号)
    Private TypeBuff As ArrayList                                      ' type   データ(第ニ項目：ブロック種別)
    Private OffsetBuff As ArrayList                                    ' Offset データ(第三項目：ブロック内相対番号)
    Private IdBuff As ArrayList                                        ' ID     データ(第四項目：データ種別)
    Private DataBuff As ArrayList                                      ' Data   データ(第五項目：データ内容)
    Private StackCount As Integer                                      ' 未保存データの数を保持します
    Private CurrentNum As Integer                                      ' 未保存データのNumberを保持します
    Private CurrentType As String                                      ' 未保存データのTypeを保持します
    Private IdStack As ArrayList                                       ' 未保存データのIDを保持します
    Private DataStack As ArrayList                                     ' 未保存データのDataを保持します

    Public Sub New()                                                   ' コンストラクタ（初期化処理）
        Debug.WriteLine("debug RegistDataクラスが作成されました。")
        Me.DataReady = CommonDefines.StatusFileUnknown                ' 変数の初期化
        Me.RegistDirectory = System.Windows.Forms.Application.StartupPath & "/files/"
        Me.RegistName = "Registry"
        Debug.WriteLine("debug RegistDirectory   [" & Me.RegistDirectory & "]")
        Debug.WriteLine("debug RegistName        [" & Me.RegistName & "]")
        Me.Buffering = False
        Me.BuffSize = 0
        Me.NumBuff = New ArrayList
        Me.TypeBuff = New ArrayList
        Me.OffsetBuff = New ArrayList
        Me.IdBuff = New ArrayList
        Me.DataBuff = New ArrayList
        Me.StackCount = 0
        Me.CurrentNum = 1
        Me.CurrentType = Nothing
        Me.IdStack = New ArrayList
        Me.DataStack = New ArrayList
        Me.ReadFile()
    End Sub

    Public Sub New(ByVal TextDir As String, ByVal TextFile As String)              ' コンストラクタ（初期化処理）
        Debug.WriteLine("debug RegistDataクラスが作成されました。")
        Me.DataReady = CommonDefines.StatusFileUnknown                ' 変数の初期化
        Me.RegistDirectory = TextDir
        Me.RegistName = TextFile
        Debug.WriteLine("debug RegistDirectory   [" & Me.RegistDirectory & "]")
        Debug.WriteLine("debug RegistName        [" & Me.RegistName & "]")
        Me.Buffering = False
        Me.BuffSize = 0
        Me.NumBuff = New ArrayList
        Me.TypeBuff = New ArrayList
        Me.OffsetBuff = New ArrayList
        Me.IdBuff = New ArrayList
        Me.DataBuff = New ArrayList
        Me.StackCount = 0
        Me.CurrentNum = 1
        Me.CurrentType = Nothing
        Me.IdStack = New ArrayList
        Me.DataStack = New ArrayList
        Me.ReadFile()
    End Sub

    Private Sub ReadFile()                                             ' 登録情報ファイルを取り出します
        Dim RegistDataFilename As String
        RegistDataFilename = Me.RegistDirectory & Me.RegistName
        Try                                                            ' 登録情報ファイルを開きます
            Debug.WriteLine("debug 登録情報ファイルの取り出しを開始します。")
            Using DataFile As StreamReader = New StreamReader(RegistDataFilename)
                Dim LineBuff As String
                Do
                    LineBuff = DataFile.ReadLine()
                    If Not String.IsNullOrEmpty(LineBuff) Then         ' 1行をタブ区切りで分割します
                        Debug.WriteLine("debug Line buff [" & LineBuff & "]")
                        Dim ColumnBuff() As String = LineBuff.Split(ControlChars.Tab)
                        Me.NumBuff.Add(ColumnBuff(0))                  ' 分割したデータをバッファに格納します
                        Me.TypeBuff.Add(ColumnBuff(1))
                        Me.OffsetBuff.Add(ColumnBuff(2))
                        Me.IdBuff.Add(ColumnBuff(3))
                        Me.DataBuff.Add(ColumnBuff(4))
                        Me.BuffSize += 1
                        Me.Buffering = True
                        Dim NumTemp As Integer = Convert.ToInt16(ColumnBuff(0)) + 1
                        If NumTemp > Me.CurrentNum Then Me.CurrentNum = NumTemp
                    End If
                Loop Until String.IsNullOrEmpty(LineBuff)
                DataFile.Close()                                       ' 登録情報ファイルを閉じます
            End Using
            Me.DataReady = CommonDefines.StatusFileReady
            Debug.WriteLine("debug 登録情報ファイルが準備できました。")
        Catch ex As DirectoryNotFoundException                         ' ディレクトリが存在しない場合
            Me.DataReady = CommonDefines.StatusFileNotDirectory
            Debug.WriteLine("debug 登録情報ファイルの取り出しに失敗しました。")
            Debug.WriteLine(ex.Message)
        Catch ex As FileNotFoundException                              ' ファイルが存在しない場合
            Me.DataReady = CommonDefines.StatusFileNotFound
            Debug.WriteLine("debug 登録情報ファイルの取り出しに失敗しました。")
            Debug.WriteLine(ex.Message)
        Catch ex As Exception                                          ' それ以外の異常が発生した場合
            Me.DataReady = CommonDefines.StatusFileNotReady
            Debug.WriteLine("debug 登録情報ファイルの取り出しに失敗しました。")
            Debug.WriteLine(ex.Message)
        End Try
    End Sub

    Public Sub StackString(ByVal IdElement As String, ByVal TextElement As String) ' String型のデータを未保存データとして追加します
        Me.IdStack.Add(IdElement)
        Me.DataStack.Add(TextElement)
        Me.StackCount += 1
        Debug.WriteLine("debug String型のデータ[" & TextElement & "]をスタックしました。")
    End Sub

    Public Sub StackLong(ByVal IdElement As String, ByVal LongElement As Long)     ' Long型のデータを未保存データとして追加します
        Me.IdStack.Add(IdElement)
        Me.DataStack.Add(LongElement.ToString())
        Me.StackCount += 1
        Debug.WriteLine("debug Long型のデータ[" & LongElement.ToString() & "]をスタックしました。")
    End Sub

    Public Sub StackArray(ByVal IdElement As String, ByVal ByteElement As Array)   ' バイナリデータを未保存データとして追加します
        Me.IdStack.Add(IdElement)
        Me.DataStack.Add(Convert.ToBase64String(ByteElement))
        Me.StackCount += 1
        Debug.WriteLine("debug バイナリデータ[" & Convert.ToBase64String(ByteElement) & "]をスタックしました。")
    End Sub

    Public Sub SetupCurrentNum(ByVal NumElement As Integer)                  ' 未保存データのブロック番号を設定します
        Me.CurrentNum = NumElement
    End Sub

    Public Sub SetupCurrentType(ByVal TypeElement As String)                 ' 未保存データのブロック種別を設定します
        Me.CurrentType = TypeElement
    End Sub

    Public Sub WriteStack()                                            ' 未保存データを登録情報ファイルに追加書きします
        Dim RegistDataFilename As String = Me.RegistDirectory & Me.RegistName
        Debug.WriteLine("debug 登録情報ファイルの書き込みを開始します。")
        If Me.DataReady <> CommonDefines.StatusFileNotReady Then
            Try                                                        ' ディレクトリが存在しない場合には作成します
                Dim CheckDir As DirectoryInfo = New DirectoryInfo(Me.RegistDirectory)
                If Not CheckDir.Exists Then
                    CheckDir.Create()
                    Debug.WriteLine("debug ディレクトリ" & Me.RegistDirectory & "を作成しました。")
                End If                                                 ' 追加書きで登録情報ファイルを開きます
                Using DataFile As StreamWriter = New StreamWriter(RegistDataFilename, True)
                    For i As Integer = 0 To Me.StackCount - 1 Step 1   ' 未保存データを書き込みます
                        Dim LineBuff As String = String.Format("{0}{1}{2}{3}{4}{5}{6}{7}{8}",
                            Me.CurrentNum, ControlChars.Tab, Me.CurrentType, ControlChars.Tab, i.ToString(), ControlChars.Tab,
                            Me.IdStack(i), ControlChars.Tab, Me.DataStack(i))
                        DataFile.WriteLine(LineBuff)
                        Debug.WriteLine("debug Line buff [" & LineBuff & "]")
                    Next
                    DataFile.Close()                                   ' 登録情報ファイルを閉じます
                End Using
                Debug.WriteLine("debug 登録情報ファイルの書き込みが正常終了しました。")
            Catch ex As Exception
                Debug.WriteLine("debug 登録情報ファイルの書き込みに失敗しました。")
                Debug.WriteLine(ex.Message)
            Finally
                Me.IdStack.Clear()                                     ' 登録情報ファイルを再読み込みします
                Me.DataStack.Clear()
                Me.StackCount = 0
                Me.Buffering = False
                Me.BuffSize = 0
                Me.NumBuff = New ArrayList
                Me.TypeBuff = New ArrayList
                Me.OffsetBuff = New ArrayList
                Me.IdBuff = New ArrayList
                Me.DataBuff = New ArrayList
                Me.StackCount = 0
                Me.CurrentNum = 1
                Me.CurrentType = Nothing
                Me.IdStack = New ArrayList
                Me.DataStack = New ArrayList
                Me.ReadFile()
            End Try
        Else
            Debug.WriteLine("debug エラーのため処理を中止しました。")
        End If
    End Sub

    Public Sub CancelStack()                                           ' 未保存データを破棄します
        Me.IdStack.Clear()
        Me.DataStack.Clear()
        Me.StackCount = 0
        Debug.WriteLine("debug 登録情報ファイルの内容が破棄されました。")
    End Sub

    Public Function GetAuthorityList() As Long()                       ' 認証局のシリアル番号リストを取得します
        Dim RetArray() As Long = {0}
        Dim Ind As Integer = 0
        For i As Integer = 0 To Me.BuffSize - 1 Step 1
            If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                If Me.TypeBuff(i) = CommonDefines.BlockCA Then
                    Array.Resize(RetArray, (Ind + 1))
                    RetArray(Ind) = Long.Parse(Me.DataBuff(i))
                    Ind += 1
                ElseIf Me.TypeBuff(i) = CommonDefines.BlockRoot Then
                    Array.Resize(RetArray, (Ind + 1))
                    RetArray(Ind) = Long.Parse(Me.DataBuff(i))
                    Ind += 1
                ElseIf Me.TypeBuff(i) = CommonDefines.BlockMiddle Then
                    Array.Resize(RetArray, (Ind + 1))
                    RetArray(Ind) = Long.Parse(Me.DataBuff(i))
                    Ind += 1
                End If
            End If
        Next
        If Ind = 0 Then
            RetArray = Nothing
        End If
        Debug.WriteLine("debug 認証局のシリアル番号は " & Ind.ToString() & " 件です。")
        Return RetArray
    End Function

    Public Function GetEndentityList() As Long()                       ' 個人・サーバー証明書のシリアル番号リストを取得します
        Dim RetArray() As Long = {0}
        Dim Ind As Integer = 0
        For i As Integer = 0 To Me.BuffSize - 1 Step 1
            If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                If Me.TypeBuff(i) = CommonDefines.BlockEndentity Then
                    Array.Resize(RetArray, (Ind + 1))
                    RetArray(Ind) = Long.Parse(Me.DataBuff(i))
                    Ind += 1
                ElseIf Me.TypeBuff(i) = CommonDefines.BlockUser Then
                    Array.Resize(RetArray, (Ind + 1))
                    RetArray(Ind) = Long.Parse(Me.DataBuff(i))
                    Ind += 1
                ElseIf Me.TypeBuff(i) = CommonDefines.BlockServer Then
                    Array.Resize(RetArray, (Ind + 1))
                    RetArray(Ind) = Long.Parse(Me.DataBuff(i))
                    Ind += 1
                End If
            End If
        Next
        If Ind = 0 Then
            RetArray = Nothing
        End If
        Debug.WriteLine("debug 個人・サーバー証明書のシリアル番号は " & Ind.ToString() & " 件です。")
        Return RetArray
    End Function

    Public Function GetCertRevoList() As Long()                        ' 失効リストのシリアル番号リストを取得します
        Dim RetArray() As Long = {0}
        Dim Ind As Integer = 0
        For i As Integer = 0 To Me.BuffSize - 1 Step 1
            If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                If Me.TypeBuff(i) = CommonDefines.BlockCRL Then
                    Array.Resize(RetArray, (Ind + 1))
                    RetArray(Ind) = Long.Parse(Me.DataBuff(i))
                    Ind += 1
                End If
            End If
        Next
        If Ind = 0 Then
            RetArray = Nothing
        End If
        Debug.WriteLine("debug 失効リストのシリアル番号は " & Ind.ToString() & " 件です。")
        Return RetArray
    End Function

    Public Function GetSerialNumberList() As Long()                    ' すべてのシリアル番号リストを取得します
        Dim RetArray() As Long = {0}
        Dim Ind As Integer = 0
        For i As Integer = 0 To Me.BuffSize - 1 Step 1
            If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                Array.Resize(RetArray, (Ind + 1))
                RetArray(Ind) = Long.Parse(Me.DataBuff(i))
                Ind += 1
            End If
        Next
        If Ind = 0 Then
            RetArray = Nothing
        End If
        Debug.WriteLine("debug すべてのシリアル番号は " & Ind.ToString() & " 件です。")
        Return RetArray
    End Function

    Public Function GetMiddleList(ByVal AuthNumber As Long) As Long()        ' 指定された認証局から発行された中間認証局証明書の
        Dim RetArray() As Long = {0}                                   '   シリアル番号リストを取得します
        Dim Target() As String = {""}
        Dim Ind As Integer = 0
        For i As Integer = 0 To Me.BuffSize - 1 Step 1                 ' シリアル番号からブロック番号を検索します
            If Me.IdBuff(i) = CommonDefines.RecordIssuer Then
                If Me.DataBuff(i) = AuthNumber.ToString() Then
                    Array.Resize(Target, (Ind + 1))
                    Target(Ind) = Me.NumBuff(i)
                    Ind += 1
                End If
            End If
        Next
        If Ind = 0 Then
            RetArray = Nothing
        Else
            Array.Resize(RetArray, Ind)
            Dim Ind2 As Integer = 0
            For j As Integer = 0 To Ind - 1 Step 1
                For i As Integer = 0 To Me.BuffSize - 1 Step 1         ' 該当するブロックのデータを取得します
                    If Target(j) = Me.NumBuff(i) Then
                        If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                            If Me.TypeBuff(i) = CommonDefines.BlockMiddle Then
                                RetArray(Ind2) = Long.Parse(Me.DataBuff(i))
                                Ind2 += 1
                            End If
                        End If
                    End If
                Next
            Next
        End If
        Debug.WriteLine("debug 認証局(" & AuthNumber.ToString() & ")から発行された中間認証局証明書のシリアル番号は " & Ind.ToString() & " 件です。")
        Return RetArray
    End Function

    Public Function GetUserList(ByVal AuthNumber As Long) As Long()          ' 指定された認証局から発行された個人証明書の
        Dim DebugLog As LogFile = New LogFile()
        Dim RetArray() As Long = {0}                                   '   シリアル番号リストを取得します
        Dim Target() As String = {""}
        Dim Ind As Integer = 0
        For i As Integer = 0 To Me.BuffSize - 1 Step 1                 ' シリアル番号からブロック番号を検索します
            If Me.IdBuff(i) = CommonDefines.RecordIssuer Then
                If Me.DataBuff(i) = AuthNumber.ToString() Then
                    Array.Resize(Target, (Ind + 1))
                    Target(Ind) = Me.NumBuff(i)
                    Ind += 1
                End If
            End If
        Next
        If Ind = 0 Then
            RetArray = Nothing
        Else
            Array.Resize(RetArray, Ind)
            Dim Ind2 As Integer = 0
            For j As Integer = 0 To Ind - 1 Step 1
                For i As Integer = 0 To Me.BuffSize - 1 Step 1         ' 該当するブロックのデータを取得します
                    If Target(j) = Me.NumBuff(i) Then
                        If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                            If Me.TypeBuff(i) = CommonDefines.BlockUser Then
                                RetArray(Ind2) = Long.Parse(Me.DataBuff(i))
                                Ind2 += 1
                            End If
                        End If
                    End If
                Next
            Next
        End If
        Debug.WriteLine("debug 認証局(" & AuthNumber.ToString() & ")から発行された個人証明書のシリアル番号は " & Ind.ToString() & " 件です。")
        Return RetArray
    End Function

    Public Function GetServerList(ByVal AuthNumber As Long) As Long()        ' 指定された認証局から発行されたサーバー証明書の
        Dim RetArray() As Long = {0}                                   '   シリアル番号リストを取得します
        Dim Target() As String = {""}
        Dim Ind As Integer = 0
        For i As Integer = 0 To Me.BuffSize - 1 Step 1                 ' シリアル番号からブロック番号を検索します
            If Me.IdBuff(i) = CommonDefines.RecordIssuer Then
                If Me.DataBuff(i) = AuthNumber.ToString() Then
                    Array.Resize(Target, (Ind + 1))
                    Target(Ind) = Me.NumBuff(i)
                    Ind += 1
                End If
            End If
        Next
        If Ind = 0 Then
            RetArray = Nothing
        Else
            Array.Resize(RetArray, Ind)
            Dim Ind2 As Integer = 0
            For j As Integer = 0 To Ind - 1 Step 1
                For i As Integer = 0 To Me.BuffSize - 1 Step 1         ' 該当するブロックのデータを取得します
                    If Target(j) = Me.NumBuff(i) Then
                        If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                            If Me.TypeBuff(i) = CommonDefines.BlockServer Then
                                RetArray(Ind2) = Long.Parse(Me.DataBuff(i))
                                Ind2 += 1
                            End If
                        End If
                    End If
                Next
            Next
        End If
        Debug.WriteLine("debug 認証局(" & AuthNumber.ToString() & ")から発行されたサーバー証明書のシリアル番号は " & Ind.ToString() & " 件です。")
        Return RetArray
    End Function

    Public Function GetCRLlist(ByVal AuthNumber As Long) As Long()           ' 指定された認証局から発行された失効リストの
        Dim RetArray() As Long = {0}                                   '   シリアル番号リストを取得します
        Dim Target() As String = {""}
        Dim Ind As Integer = 0
        For i As Integer = 0 To Me.BuffSize - 1 Step 1                 ' シリアル番号からブロック番号を検索します
            If Me.IdBuff(i) = CommonDefines.RecordIssuer Then
                If Me.DataBuff(i) = AuthNumber.ToString() Then
                    Array.Resize(Target, (Ind + 1))
                    Target(Ind) = Me.NumBuff(i)
                    Ind += 1
                End If
            End If
        Next
        If Ind = 0 Then
            RetArray = Nothing
        Else
            Array.Resize(RetArray, Ind)
            Dim Ind2 As Integer = 0
            For j As Integer = 0 To Ind - 1 Step 1
                For i As Integer = 0 To Me.BuffSize - 1 Step 1         ' 該当するブロックのデータを取得します
                    If Target(j) = Me.NumBuff(i) Then
                        If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                            If Me.TypeBuff(i) = CommonDefines.BlockCRL Then
                                RetArray(Ind2) = Long.Parse(Me.DataBuff(i))
                                Ind2 += 1
                            End If
                        End If
                    End If
                Next
            Next
        End If
        Debug.WriteLine("debug 認証局(" & AuthNumber.ToString() & ")から発行された失効リストのシリアル番号は " & Ind.ToString() & " 件です。")
        Return RetArray
    End Function

    Public Function GetCertType(ByVal SerialNumber As Long) As String        ' ブロック種別を取得します
        Dim RetText As String = Nothing
        Dim Target As String = Nothing
        For i As Integer = 0 To Me.BuffSize - 1 Step 1                 ' シリアル番号からブロック種別を取得します
            If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                If Me.DataBuff(i) = SerialNumber.ToString() Then
                    RetText = Me.TypeBuff(i)
                    Exit For
                End If
            End If
        Next
        Debug.WriteLine("debug シリアル番号 [" & SerialNumber.ToString() & "] ブロック種別 [" & RetText & "] を取得しました。")
        Return RetText
    End Function

    Public Function GetString(ByVal SerialNumber As Long, ByVal IdElement As String) As String
        Dim RetText As String = Nothing                                ' 登録情報ファイルの内容をString型で取得します
        Dim Target As String = Nothing
        For i As Integer = 0 To Me.BuffSize - 1 Step 1                 ' シリアル番号からブロック番号を検索します
            If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                If Me.DataBuff(i) = SerialNumber.ToString() Then
                    Target = Me.NumBuff(i)
                    Exit For
                End If
            End If
        Next
        If Target IsNot Nothing Then
            For i As Integer = 0 To Me.BuffSize - 1 Step 1             ' 該当するブロックのデータを取得します
                If Target = Me.NumBuff(i) Then
                    If IdElement = Me.IdBuff(i) Then
                        RetText = Me.DataBuff(i)
                    End If
                End If
            Next
        End If
        Debug.WriteLine("debug シリアル番号 [" & SerialNumber.ToString() & "] データ種別 [" & IdElement & "] データ [" & RetText & "] を取得しました。")
        Return RetText
    End Function

    Public Function GetLong(ByVal SerialNumber As Long, ByVal IdElement As String) As Long
        Dim RetValue As Long = 0                                       ' 登録情報ファイルの内容をLong型で取得します
        Dim Target As String = Nothing
        For i As Integer = 0 To Me.BuffSize - 1 Step 1                 ' シリアル番号からブロック番号を検索します
            If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                If Me.DataBuff(i) = SerialNumber.ToString() Then
                    Target = Me.NumBuff(i)
                    Exit For
                End If
            End If
        Next
        If Target IsNot Nothing Then
            For i As Integer = 0 To Me.BuffSize - 1 Step 1             ' 該当するブロックのデータを取得します
                If Target = Me.NumBuff(i) Then
                    If IdElement = Me.IdBuff(i) Then
                        RetValue = Long.Parse(Me.DataBuff(i))
                    End If
                End If
            Next
        End If
        Debug.WriteLine("debug シリアル番号 [" & SerialNumber.ToString() & "] データ種別 [" & IdElement & "] データ [" & RetValue.ToString() & "] を取得しました。")
        Return RetValue
    End Function

    Public Function GetArray(ByVal SerialNumber As Long, ByVal IdElement As String) As Array
        Dim RetArray As Array = Nothing                                ' 登録情報ファイルの内容をバイナリデータで取得します
        Dim Target As String = Nothing
        For i As Integer = 0 To Me.BuffSize - 1 Step 1                 ' シリアル番号からブロック番号を検索します
            If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                If Me.DataBuff(i) = SerialNumber.ToString() Then
                    Target = Me.NumBuff(i)
                    Exit For
                End If
            End If
        Next
        If Target IsNot Nothing Then
            For i As Integer = 0 To Me.BuffSize - 1 Step 1             ' 該当するブロックのデータを取得します
                If Target = Me.NumBuff(i) Then
                    If IdElement = Me.IdBuff(i) Then
                        RetArray = Convert.FromBase64String(Me.DataBuff(i))
                    End If
                End If
            Next
        End If
        Debug.WriteLine("debug シリアル番号 [" & SerialNumber.ToString() & "] データ種別 [" & IdElement & "] データ [" & RetArray.ToString() & "] を取得しました。")
        Return RetArray
    End Function

    Public Sub SetRevocate(ByVal SerialNumber As Long, ByVal RevoCode As Integer, ByVal RevoDate As Date) ' 失効情報を追記します
        Dim TargetNum As String = Nothing
        Dim TargetTyp As String = Nothing
        For i As Integer = 0 To Me.BuffSize - 1 Step 1                 ' シリアル番号からブロック番号・種別を検索します
            If Me.IdBuff(i) = CommonDefines.RecordSirinumber Then
                If Me.DataBuff(i) = SerialNumber.ToString() Then
                    TargetNum = Me.NumBuff(i)
                    TargetTyp = Me.TypeBuff(i)
                End If
            End If
        Next
        If (TargetNum IsNot Nothing) And (TargetTyp IsNot Nothing) Then  ' 該当するブロック番号・種別で失効情報を追記します
            Dim Target As Integer = Me.StackCount
            Me.IdStack.Add(CommonDefines.RecordReasonCode)
            Me.DataStack.Add(RevoCode.ToString())
            Me.StackCount += 1
            Me.IdStack.Add(CommonDefines.RecordRevocate)
            Me.DataStack.Add(RevoDate.ToString("yyyy/MM/dd 00:00:00"))
            Me.StackCount += 1
            Me.CurrentNum = Convert.ToInt16(TargetNum)
            Me.CurrentType = TargetTyp
            Debug.WriteLine("debug シリアル番号 [" & SerialNumber.ToString() & "] 失効理由 [" & RevoCode.ToString() & "] 失効しました。")
        End If
    End Sub
End Class
