﻿' * 名前 : k9piaCertImport.vb
' * 機能 : 発行局とのインターフェース
' * 作成 : 2014/04/-- Ver1.0
' * 更新 : ----/--/-- Ver -.-
' * 備考 : XMLインターフェースを使用して、発行局に証明書のインポートを指示します。

Imports Microsoft.VisualBasic.ControlChars
Imports System.IO
Imports System.Text
Imports System.Diagnostics

Public Class k9piaCertImport
    Public Property K9piaExecDirectory As String                       ' 発行局の実行ディレクトリを保持します
    Public Property K9piaExecName As String                            ' 発行局の実行ファイル名を保持します
    Public Property CertFilename As String                             ' 証明書のファイル名(ディレクトリ含む)を保持します
    Public Property CertExtend As String                               ' 証明書のファイルの拡張子を保持します
    Public Property CertPassword As String                             ' 証明書のパスワードを保持します
    Public Property JPKIused As Boolean                                ' JPKIを使用するか否かを保持します

    Public Sub New()                                                   ' コンストラクタ
        Me.K9piaExecDirectory = System.Windows.Forms.Application.StartupPath & "/bin/"
        Me.K9piaExecName = "k9pia.exe"
        Me.CertFilename = Nothing
        Me.CertExtend = Nothing
        Me.CertPassword = Nothing
        Me.JPKIused = False
    End Sub

    Public Function sendCommand() As Integer                           ' 証明書をインポートします
        Dim RetCode As Integer
        Dim XmlLine As ArrayList = New ArrayList                       ' インポート用のXMLファイルを作成します
        Dim K9piaTemp As String = Me.K9piaExecDirectory & "k9pia_ImportCert.xml"
        XmlLine.Add("<?xml version=""1.0"" encoding=""UTF-8""?>")
        XmlLine.Add("<beginCertificate action=""import"">")
        XmlLine.Add("  <input>")
        XmlLine.Add("    <filename extend=""" & Me.CertExtend & """>" & Me.CertFilename & "</filename>")
        If Not String.IsNullOrEmpty(Me.CertPassword) Then
            XmlLine.Add("    <password>" & Me.CertPassword & "</password>")
        End If
        XmlLine.Add("  </input>")
        If Me.JPKIused Then
            XmlLine.Add("  <subjectPrivatekey key=""apply"">")
            XmlLine.Add("    <cryptographicProviderNames ProviderType=""" & CommonDefines.K9cspJpkType & """>")
            XmlLine.Add("      " & CommonDefines.K9cspJpk)
            XmlLine.Add("    </cryptographicProviderNames>")
            XmlLine.Add("    <HSMtype>JPKI</HSMtype>")
            XmlLine.Add("  </subjectPrivatekey>")
        End If
        XmlLine.Add("</beginCertificate>")

        Try                                                            ' XMLを一時ファイルに書き込みます
            Using DataFile As StreamWriter = New StreamWriter(K9piaTemp, False)
                For i As Integer = 0 To XmlLine.Count - 1 Step 1
                    DataFile.WriteLine(XmlLine(i))
                Next
                DataFile.Close()
            End Using
            Debug.WriteLine("XMLファイルを作成しました。")
            Debug.WriteLine("[" & K9piaTemp & "]")
        Catch ex As Exception
            Debug.WriteLine("XMLファイルの作成に失敗しました。")
            Debug.WriteLine("[" & K9piaTemp & "]")
            Debug.WriteLine(ex.Message)
            RetCode = ErrorCodeK9piaXml
        End Try

        If RetCode = 0 Then
            Try                                                        ' 発行局へ証明書インポートコマンドを送ります
                Dim ChildCode As String
                Dim TimeoutFlag As Boolean
                Dim TimeoutMillisec As Integer = 600 * 1000
                Dim CommandLine As String = String.Format("{0}{1}", Me.K9piaExecDirectory, Me.K9piaExecName)
                Dim Parame1Line As String = String.Format("""{0}""", K9piaTemp)
                Dim ExecK9pia As System.Diagnostics.Process            ' コンソール画面をワザと表示させています
                ExecK9pia = System.Diagnostics.Process.Start(CommandLine, Parame1Line)
                ' Dim ProcInfo As New ProcessStartInfo(CommandLine, Parame1Line)
                ' ProcInfo.CreateNoWindow = True
                ' ProcInfo.UseShellExecute = False
                ' ExecK9pia = System.Diagnostics.Process.Start(ProcInfo)
                TimeoutFlag = ExecK9pia.WaitForExit(TimeoutMillisec)
                If TimeoutFlag Then
                    ChildCode = ExecK9pia.ExitCode
                    If ChildCode = 0 Then                              ' 発行局が正常終了した場合
                        Debug.WriteLine("証明書を発行しました。")
                        RetCode = 0
                        System.IO.File.Delete(K9piaTemp)               ' インポート用XMLファイルを消去します
                    Else                                               ' 発行局がエラー終了した場合
                        Debug.WriteLine("証明書の発行に失敗しました。")
                        Debug.WriteLine("終了コード " & ChildCode.ToString())
                        RetCode = ChildCode
                        ' System.IO.File.Delete(K9piaTemp)             ' インポート用XMLファイルを消去します
                    End If
                Else                                                   ' 発行局がタイムアウトした場合
                    Debug.WriteLine("証明書の発行でタイムアウトが発生しました。")
                    RetCode = ErrorCodeK9piaTimeout
                    ' System.IO.File.Delete(K9piaTemp)                 ' インポート用XMLファイルを消去します
                End If
            Catch ex As Exception                                      ' その他の異常の場合
                Debug.WriteLine("証明書の発行に失敗しました。")
                Debug.WriteLine(ex.Message)
                RetCode = ErrorCodeK9piaCreate
                ' System.IO.File.Delete(K9piaTemp)                     ' インポート用XMLファイルの消去
            End Try
        End If
        Return RetCode
    End Function
End Class
