﻿' * 名前 : k9piaCertIssue.vb
' * 機能 : 発行局とのインターフェース
' * 作成 : 2014/04/-- Ver1.0
' * 更新 : ----/--/-- Ver -.-
' * 備考 : XMLインターフェースを使用して、発行局に証明書の発行を指示します。
' *        本インターフェースは暗号化などを行うため時間がかかることがあります。
' *        このため、本インターフェースの呼び出し元でAsync/Awaitを指定する必要があります。
' *        XMLファイルの詳細は「40 発行局XMLインターフェース仕様書.pdf」を参照してください。
' *        補足：発行局へのインターフェースは・・・、ベタです 。d(￣ ￣)　ヾ(^o^;
' *        イヤァ、だってその方が分かりやすいでしょ。
' *        理解してもらって、流用・応用してもらうのが目的なんだし。

Imports Microsoft.VisualBasic.ControlChars
Imports System.IO
Imports System.Text
Imports System.Diagnostics

Public Class k9piaCertIssue
    Public Property K9piaExecDirectory As String                       ' 発行局の実行ディレクトリを保持します
    Public Property K9piaExecName As String                            ' 発行局の実行ファイル名を保持します
    Public Property SerialNumber As Long                               ' 証明書のシリアル番号を保持します
    Public Property SignatureAlgorithm As String                       ' 証明書の署名アルゴリズムを保持します
    Public Property Issuer As Long                                     ' 証明書の発行者を保持します
    Public Property NotBefore As Date                                  ' 証明書の有効期限開始日を保持します
    Public Property NotAfter As Date                                   ' 証明書の有効期限終了日を保持します
    Public Property Country As String                                  ' 証明書の国を保持します
    Public Property StateOrProvince As String                          ' 証明書の都道府県を保持します
    Public Property Locality As String                                 ' 証明書の市区町村を保持します
    Public Property Organization As String                             ' 証明書の組織を保持します
    Public Property OrganizationUnit1 As String                        ' 証明書の部署を保持します
    Public Property OrganizationUnit2 As String                        ' 証明書の部署を保持します
    Public Property OrganizationUnit3 As String                        ' 証明書の部署を保持します
    Public Property CommonName As String                               ' 証明書の固有名称を保持します
    Public Property Email As String                                    ' 証明書のメールアドレスを保持します
    Public Property Postal As String                                   ' 証明書の郵便番号を保持します
    Public Property Street As String                                   ' 証明書の番地を保持します
    Public Property JIcountry As String                                ' 証明書の法人登記所の国を保持します
    Public Property JIstateOrProvince As String                        ' 証明書の法人登記所の都道府県を保持します
    Public Property JIlocality As String                               ' 証明書の法人登記所の市区町村を保持します
    Public Property JInumber As String                                 ' 証明書の法人番号を保持します
    Public Property BasicConst As Boolean                              ' 証明書が認証局か否かを保持します
    Public Property PathLen As Integer                                 ' 証明書が認証局の時、パスの長さを保持します
    Public Property PolicyOID As String                                ' 証明書のポリシーOIDを保持します
    Public Property PolicyURI As String                                ' 証明書のCPSのURI(http～)またはメッセージ(http以外)を保持します
    Public Property CrlDistribution As String                          ' 証明書の失効リスト配布場所を保持します
    Public Property AuthInfo As String                                 ' 証明書の認証局アクセス情報を保持します
    Public Property ImageFilename As String                            ' 証明書に埋め込む画像のファイル名(ディレクトリ含む)を保持します
    Public Property withCert1 As Long                                  ' 同梱する証明書を保持します
    Public Property withCert2 As Long                                  ' 同梱する証明書を保持します
    Public Property withCert3 As Long                                  ' 同梱する証明書を保持します
    Public Property ProviderType As String                             ' 暗号プロバイダの種別を保持します
    Public Property ProviderName As String                             ' 暗号プロバイダの名前を保持します
    Public Property Keysize As Integer                                 ' 秘密鍵の鍵長を保持します
    Public Property CertFilename As String                             ' 証明書のファイル名(ディレクトリ含む)を保持します
    Public Property CertExtend As String                               ' 証明書のファイルの拡張子を保持します
    Public Property CertPassword As String                             ' 証明書のパスワードを保持します
    Public Property SSLrequire As Integer                              ' 証明書と共にSSLrequire用ファイルを出力するか否かを保持します
    Public Property CsrFilename As String                              ' 証明書の作成要求ファイル名(ディレクトリ含む)を保持します

    Public Sub New()                                                   ' コンストラクタ
        Me.K9piaExecDirectory = System.Windows.Forms.Application.StartupPath & "/bin/"
        Me.K9piaExecName = "k9pia.exe"
        Me.SerialNumber = 0
        Me.SignatureAlgorithm = Nothing
        Me.Issuer = 0
        Me.NotBefore = Nothing
        Me.NotAfter = Nothing
        Me.Country = Nothing
        Me.StateOrProvince = Nothing
        Me.Locality = Nothing
        Me.Organization = Nothing
        Me.OrganizationUnit1 = Nothing
        Me.OrganizationUnit2 = Nothing
        Me.OrganizationUnit3 = Nothing
        Me.CommonName = Nothing
        Me.Email = Nothing
        Me.Postal = Nothing
        Me.Street = Nothing
        Me.JIcountry = Nothing
        Me.JIstateOrProvince = Nothing
        Me.JIlocality = Nothing
        Me.JInumber = Nothing
        Me.BasicConst = False
        Me.PathLen = -1
        Me.PolicyOID = Nothing
        Me.PolicyURI = Nothing
        Me.CrlDistribution = Nothing
        Me.AuthInfo = Nothing
        Me.ImageFilename = Nothing
        Me.withCert1 = 0
        Me.withCert2 = 0
        Me.withCert3 = 0
        Me.ProviderType = Nothing
        Me.ProviderName = Nothing
        Me.Keysize = 0
        Me.CertFilename = Nothing
        Me.CertExtend = Nothing
        Me.CertPassword = Nothing
        Me.SSLrequire = 0
        Me.CsrFilename = Nothing
    End Sub

    Public Function sendCommand() As Integer                           ' 証明書を発行します
        Dim RetCode As Integer
        Dim XmlLine As ArrayList = New ArrayList                       ' 発行用のXMLファイルを作成します
        Dim K9piaTemp As String = Me.K9piaExecDirectory & "k9pia_CreateCert.xml"
        XmlLine.Add("<?xml version=""1.0"" encoding=""UTF-8""?>")
        XmlLine.Add("<beginCertificate action=""issue"">")
        XmlLine.Add("  <certificate>")                                 ' 証明書の内容を定義します
        XmlLine.Add("    <version>v3</version>")                       ' -> 証明書のバージョンと署名アルゴリズム
        XmlLine.Add("    <serialNumber>" & Me.SerialNumber.ToString() & "</serialNumber>")
        XmlLine.Add("    <signatureAlgorithm>" & Me.SignatureAlgorithm & "</signatureAlgorithm>")
        If Me.Issuer = 0 Then                                          ' -> 証明書の発行者
            XmlLine.Add("    <issuer>mySelf</issuer>")
        Else
            XmlLine.Add("    <issuer>" & Me.Issuer.ToString() & "</issuer>")
        End If
        XmlLine.Add("    <validity timezone=""JST"">")                 ' -> 証明書の有効期限
        XmlLine.Add("      <notBefore>" & Me.NotBefore & "</notBefore>")
        XmlLine.Add("      <notAfter>" & Me.NotAfter & "</notAfter>")
        XmlLine.Add("    </validity>")
        XmlLine.Add("    <subject>")                                   ' -> 証明書の所有者
        XmlLine.Add("      <countryName>" & Me.Country & "</countryName>")
        If Not String.IsNullOrEmpty(Me.StateOrProvince) Then
            XmlLine.Add("      <stateOrProvinceName>" & Me.StateOrProvince & "</stateOrProvinceName>")
        End If
        If Not String.IsNullOrEmpty(Me.Locality) Then
            XmlLine.Add("      <localityName>" & Me.Locality & "</localityName>")
        End If
        XmlLine.Add("      <organizationName>" & Me.Organization & "</organizationName>")
        If Not String.IsNullOrEmpty(Me.OrganizationUnit1) Then
            XmlLine.Add("      <organizationUnitName>" & Me.OrganizationUnit1 & "</organizationUnitName>")
        End If
        If Not String.IsNullOrEmpty(Me.OrganizationUnit2) Then
            XmlLine.Add("      <organizationUnitName>" & Me.OrganizationUnit2 & "</organizationUnitName>")
        End If
        If Not String.IsNullOrEmpty(Me.OrganizationUnit3) Then
            XmlLine.Add("      <organizationUnitName>" & Me.OrganizationUnit3 & "</organizationUnitName>")
        End If
        If Not String.IsNullOrEmpty(Me.Email) Then
            XmlLine.Add("      <eMail>" & Me.Email & "</eMail>")
        End If
        If Not String.IsNullOrEmpty(Me.Postal) Then
            XmlLine.Add("      <postalCode>" & Me.Postal & "</postalCode>")
        End If
        If Not String.IsNullOrEmpty(Me.Street) Then
            XmlLine.Add("      <streetAddress>" & Me.Street & "</streetAddress>")
        End If
        XmlLine.Add("      <commonName>" & Me.CommonName & "</commonName>")
        If Not String.IsNullOrEmpty(Me.JIcountry) Then
            XmlLine.Add("      <jurisdictionOfIncorporationCountryName>" & Me.JIcountry & "</jurisdictionOfIncorporationCountryName>")
        End If
        If Not String.IsNullOrEmpty(Me.JIstateOrProvince) Then
            XmlLine.Add("      <jurisdictionOfIncorporationStateOrProvinceName>" & Me.JIstateOrProvince & "</jurisdictionOfIncorporationStateOrProvinceName>")
        End If
        If Not String.IsNullOrEmpty(Me.JIlocality) Then
            XmlLine.Add("      <jurisdictionOfIncorporationLocalityName>" & Me.JIlocality & "</jurisdictionOfIncorporationLocalityName>")
        End If
        If Not String.IsNullOrEmpty(Me.JInumber) Then
            XmlLine.Add("      <jurisdictionOfIncorporationNumber>" & Me.JInumber & "</jurisdictionOfIncorporationNumber>")
        End If
        XmlLine.Add("    </subject>")
        XmlLine.Add("    <subjectPublickeyInfo />")                    ' -> 所有者の公開鍵
        XmlLine.Add("    <extensions>")                                ' -> 証明書の拡張領域
        XmlLine.Add("      <subjectKeyIdentifier>")                    ' -> 公開鍵識別子
        XmlLine.Add("        <subjectKeyHash method=""SHA-1"" />")
        XmlLine.Add("      </subjectKeyIdentifier>")
        XmlLine.Add("      <authorityKeyIdentifier>")                  ' -> 認証局識別子 
        XmlLine.Add("        <issuerKeyHash method=""SHA-1"" />")
        XmlLine.Add("        <issuerSubject />")
        XmlLine.Add("        <issuerSerialNumber />")
        XmlLine.Add("      </authorityKeyIdentifier>")
        If Me.BasicConst Then
            XmlLine.Add("      <basicConstraints critical=""critical"">")
            XmlLine.Add("        <cA>CA</cA>")                         ' -> 基本制限(認証局)
            If Me.PathLen >= 0 Then
                XmlLine.Add("        <pathLenConstraint>" & Me.PathLen.ToString() & "</pathLenConstraint>")
            End If
            XmlLine.Add("      </basicConstraints>")
            XmlLine.Add("      <keyUsage critical=""non-critical"">")  ' -> 鍵使用法(認証局)
            XmlLine.Add("        digitalSignature,keyEncipherment,dataEncipherment,keyCertSign,cRLSign")
            XmlLine.Add("      </keyUsage>")
        Else                                                           ' -> 基本制限(個人／サーバー)
            XmlLine.Add("      <basicConstraints critical=""critical"">")
            XmlLine.Add("        <cA>endEntity</cA>")
            XmlLine.Add("      </basicConstraints>")
        End If
        If Not String.IsNullOrEmpty(Me.PolicyURI) Then
            XmlLine.Add("      <certificatePolicies>")                 ' -> 証明書ポリシー
            XmlLine.Add("        <policyInfo>")
            XmlLine.Add("          <policyOID>" & Me.PolicyOID & "</policyOID>")
            If Left(Me.PolicyURI, 4) = "http" Then
                XmlLine.Add("          <CPSuri>" & Me.PolicyURI & "</CPSuri>")
            Else
                XmlLine.Add("          <userNotice>" & Me.PolicyURI & "</userNotice>")
            End If
            XmlLine.Add("        </policyInfo>")
            XmlLine.Add("      </certificatePolicies>")
        End If
        If Not String.IsNullOrEmpty(Me.CrlDistribution) Then
            XmlLine.Add("      <cRLDistributionPoints>")               ' -> 失効リスト配布場所
            XmlLine.Add("        <distUrl>" & Me.CrlDistribution & "</distUrl>")
            XmlLine.Add("      </cRLDistributionPoints>")
        End If
        If Not String.IsNullOrEmpty(Me.AuthInfo) Then
            XmlLine.Add("      <AuthorityInfoAccess>")                 ' -> 認証局情報
            XmlLine.Add("        <caIssuer>" & Me.AuthInfo & "</caIssuer>")
            XmlLine.Add("      </AuthorityInfoAccess>")
        End If
        If Not String.IsNullOrEmpty(Me.ImageFilename) Then
            XmlLine.Add("      <originalData>")                        ' -> 画像データ
            XmlLine.Add("        <sequence>")
            XmlLine.Add("          <oid>0.2.440.200280.100</oid>")
            XmlLine.Add("          <boolean>false</boolean>")
            XmlLine.Add("          <octetString>")
            XmlLine.Add("            <sequence>")
            Dim IndFilenm As Integer = Me.ImageFilename.LastIndexOf("/")
            If IndFilenm < 0 Then
                IndFilenm = Me.ImageFilename.LastIndexOf("\")
                If IndFilenm < 0 Then
                    IndFilenm = Me.ImageFilename.LastIndexOf(":")
                    If IndFilenm < 0 Then
                        IndFilenm = 0
                    Else
                        IndFilenm += 1
                    End If
                Else
                    IndFilenm += 1
                End If
            Else
                IndFilenm += 1
            End If
            Dim ImageFilenameTemp As String = Me.ImageFilename.Substring(IndFilenm)
            XmlLine.Add("              <sequence>")
            XmlLine.Add("                <oid>0.2.440.200280.100.100</oid>")
            XmlLine.Add("                <utf8String>" & ImageFilenameTemp & "</utf8String>")
            XmlLine.Add("              </sequence>")
            Dim ImageType As String = ""
            Try
                Dim CheckReader As BinaryReader = New BinaryReader(File.Open(Me.ImageFilename, FileMode.Open))
                Dim BinaryBuff As Byte() = CheckReader.ReadBytes(16)
                CheckReader.Close()
                If ((BinaryBuff(6) = 74) And (BinaryBuff(7) = 70) And (BinaryBuff(8) = 73) And (BinaryBuff(9) = 70)) Then
                    ImageType = "JPEG"
                ElseIf ((BinaryBuff(0) = 71) And (BinaryBuff(1) = 73) And (BinaryBuff(2) = 70)) Then
                    ImageType = "GIF"
                ElseIf ((BinaryBuff(1) = 80) And (BinaryBuff(2) = 78) And (BinaryBuff(3) = 71)) Then
                    ImageType = "PNG"
                ElseIf ((BinaryBuff(0) = 66) And (BinaryBuff(1) = 77)) Then
                    ImageType = "BM"
                Else
                    ImageType = "unknown"
                End If
            Catch ex As Exception
                ImageType = "Exception error"
            End Try
            XmlLine.Add("              <sequence>")
            XmlLine.Add("                <oid>0.2.440.200280.100.120</oid>")
            XmlLine.Add("                <printableString>" & ImageType & "</printableString>")
            XmlLine.Add("              </sequence>")
            XmlLine.Add("              <sequence>")
            XmlLine.Add("                <oid>0.2.440.200280.100.201</oid>")
            XmlLine.Add("                <bitString format=""file"">")
            XmlLine.Add("                  0," & ImageFilename)
            XmlLine.Add("                </bitString>")
            XmlLine.Add("              </sequence>")
            XmlLine.Add("            </sequence>")
            XmlLine.Add("          </octetString>")
            XmlLine.Add("        </sequence>")
            XmlLine.Add("      </originalData>")
        End If
        XmlLine.Add("    </extensions>")
        XmlLine.Add("  </certificate>")
        If (Me.withCert1 <> 0) Or (Me.withCert2 <> 0) Or (Me.withCert3 <> 0) Then
            XmlLine.Add("  <withCertificate>")                          ' -> 同梱証明書
            If Me.withCert1 <> 0 Then
                XmlLine.Add("    <serialNumber>" & Me.withCert1.ToString() & "</serialNumber>")
            End If
            If Me.withCert2 <> 0 Then
                XmlLine.Add("    <serialNumber>" & Me.withCert2.ToString() & "</serialNumber>")
            End If
            If Me.withCert3 <> 0 Then
                XmlLine.Add("    <serialNumber>" & Me.withCert3.ToString() & "</serialNumber>")
            End If
            XmlLine.Add("  </withCertificate>")
        End If
        If String.IsNullOrEmpty(Me.CsrFilename) Then
            If Me.ProviderName = CommonDefines.K9cspDef Then
                XmlLine.Add("  <subjectPrivatekey key=""create"">")        ' -> 秘密鍵の定義
                XmlLine.Add("    <cryptographicProviderNames ProviderType=""" & CommonDefines.K9cspDefType & """>")
                XmlLine.Add("      " & CommonDefines.K9cspDef)
                XmlLine.Add("    </cryptographicProviderNames>")
                XmlLine.Add("    <keySize>" & Me.Keysize.ToString() & "</keySize>")
                XmlLine.Add("  </subjectPrivatekey>")
            ElseIf Me.ProviderName = CommonDefines.K9cspAes Then
                XmlLine.Add("  <subjectPrivatekey key=""create"">")
                XmlLine.Add("    <cryptographicProviderNames ProviderType=""" & CommonDefines.K9cspAesType & """>")
                XmlLine.Add("      " & CommonDefines.K9cspAes)
                XmlLine.Add("    </cryptographicProviderNames>")
                XmlLine.Add("    <keySize>" & Me.Keysize.ToString() & "</keySize>")
                XmlLine.Add("  </subjectPrivatekey>")
            ElseIf Me.ProviderName = CommonDefines.K9cspAXP Then
                XmlLine.Add("  <subjectPrivatekey key=""create"">")
                XmlLine.Add("    <cryptographicProviderNames ProviderType=""" & CommonDefines.K9cspAXPType & """>")
                XmlLine.Add("      " & CommonDefines.K9cspAXP)
                XmlLine.Add("    </cryptographicProviderNames>")
                XmlLine.Add("    <keySize>" & Me.Keysize.ToString() & "</keySize>")
                XmlLine.Add("  </subjectPrivatekey>")
            Else
                XmlLine.Add("  <subjectPrivatekey key=""apply"">")
                XmlLine.Add("    <cryptographicProviderNames ProviderType=""" & CommonDefines.K9cspJpkType & """>")
                XmlLine.Add("      " & Me.ProviderName)
                XmlLine.Add("    </cryptographicProviderNames>")
                XmlLine.Add("    <HSMtype>JPKI</HSMtype>")
                XmlLine.Add("  </subjectPrivatekey>")
            End If
        Else
            XmlLine.Add("  <csr format=""file"">")
            XmlLine.Add("    " & Me.CsrFilename)
            XmlLine.Add("  </csr>")
        End If
        If Me.SSLrequire = CommonDefines.K9sslApache Then
            XmlLine.Add("  <output with=""SSLRequire"">")              ' -> 出力ファイル
        ElseIf Me.SSLrequire = CommonDefines.K9sslIIS7 Then
            XmlLine.Add("  <output with=""MAP7"">")
        Else
            XmlLine.Add("  <output>")
        End If
        XmlLine.Add("    <filename extend=""" & Me.CertExtend & """>" & Me.CertFilename & "</filename>")
        If Not String.IsNullOrEmpty(Me.CertPassword) Then
            XmlLine.Add("    <password>" & Me.CertPassword & "</password>")
        End If
        XmlLine.Add("  </output>")
        XmlLine.Add("</beginCertificate>")

        Try                                                            ' XMLを一時ファイルに書き込みます
            Using DataFile As StreamWriter = New StreamWriter(K9piaTemp, False)
                For i As Integer = 0 To XmlLine.Count - 1 Step 1
                    DataFile.WriteLine(XmlLine(i))
                Next
                DataFile.Close()
            End Using
            Debug.WriteLine("XMLファイルを作成しました。")
            Debug.WriteLine("[" & K9piaTemp & "]")
        Catch ex As Exception
            Debug.WriteLine("XMLファイルの作成に失敗しました。")
            Debug.WriteLine("[" & K9piaTemp & "]")
            Debug.WriteLine(ex.Message)
            RetCode = ErrorCodeK9piaXml
        End Try

        If RetCode = 0 Then
            Try                                                        ' 発行局へ証明書発行コマンドを送ります
                Dim ChildCode As String
                Dim TimeoutFlag As Boolean
                Dim TimeoutMillisec As Integer = 600 * 1000
                Dim CommandLine As String = String.Format("{0}{1}", Me.K9piaExecDirectory, Me.K9piaExecName)
                Dim Parame1Line As String = String.Format("""{0}""", K9piaTemp)
                Dim ExecK9pia As System.Diagnostics.Process            ' コンソール画面をワザと表示させています
                ExecK9pia = System.Diagnostics.Process.Start(CommandLine, Parame1Line)
                ' Dim ProcInfo As New ProcessStartInfo(CommandLine, Parame1Line)
                ' ProcInfo.CreateNoWindow = True
                ' ProcInfo.UseShellExecute = False
                ' ExecK9pia = System.Diagnostics.Process.Start(ProcInfo)
                TimeoutFlag = ExecK9pia.WaitForExit(TimeoutMillisec)
                If TimeoutFlag Then
                    ChildCode = ExecK9pia.ExitCode
                    If ChildCode = 0 Then                              ' 発行局が正常終了した場合
                        Debug.WriteLine("証明書を発行しました。")
                        RetCode = 0
                        System.IO.File.Delete(K9piaTemp)               ' 発行用XMLファイルを消去します
                    Else                                               ' 発行局がエラー終了した場合
                        Debug.WriteLine("証明書の発行に失敗しました。")
                        Debug.WriteLine("終了コード " & ChildCode.ToString())
                        RetCode = ChildCode
                        ' System.IO.File.Delete(K9piaTemp)             ' 発行用XMLファイルを消去します
                        Dim LOGdata As LogFile = New LogFile()
                        LOGdata.Logout("証明書の発行に失敗しました。")
                        LOGdata.Logout("終了コード " & ChildCode.ToString())
                        LOGdata.Logout("発行用XMLファイル " & K9piaTemp)
                    End If
                Else                                                   ' 発行局がタイムアウトした場合
                    Debug.WriteLine("証明書の発行でタイムアウトが発生しました。")
                    RetCode = ErrorCodeK9piaTimeout
                    System.IO.File.Delete(K9piaTemp)                   ' 発行用XMLファイルを消去します
                End If
            Catch ex As Exception                                      ' その他の異常の場合
                Debug.WriteLine("証明書の発行に失敗しました。")
                Debug.WriteLine(ex.Message)
                RetCode = ErrorCodeK9piaCreate
                System.IO.File.Delete(K9piaTemp)                       ' 発行用XMLファイルの消去
            End Try
        End If
        Return RetCode
    End Function
End Class
