﻿' * 名前 : k9piaCertRevocate.vb
' * 機能 : 発行局とのインターフェース
' * 作成 : 2014/04/-- Ver1.0
' * 更新 : ----/--/-- Ver -.-
' * 備考 : XMLインターフェースを使用して、発行局に証明書の失効を指示します。
' *

Imports Microsoft.VisualBasic.ControlChars
Imports System.IO
Imports System.Text
Imports System.Diagnostics

Public Class k9piaCertRevocate
    Public Property K9piaExecDirectory As String                       ' 発行局の実行ディレクトリを保持します
    Public Property K9piaExecName As String                            ' 発行局の実行ファイル名を保持します
    Public Property SerialNumber As Long                               ' 証明書のシリアル番号を保持します
    Public Property ReasonCode As Integer                              ' 証明書の失効理由を保持します
    Public Property RevocateDate As Date                               ' 証明書の失効日を保持します

    Public Sub New()                                                   ' コンストラクタ
        Me.K9piaExecDirectory = System.Windows.Forms.Application.StartupPath & "/bin/"
        Me.K9piaExecName = "k9pia.exe"
        Me.SerialNumber = 0
        Me.ReasonCode = 0
        Me.RevocateDate = Nothing
    End Sub

    Public Function sendCommand() As Integer                           ' 証明書を発行します
        Dim RetCode As Integer
        Dim XmlLine As ArrayList = New ArrayList                       ' 発行用のXMLファイルを作成します
        Dim K9piaTemp As String = Me.K9piaExecDirectory & "k9pia_RevocateCert.xml"
        XmlLine.Add("<?xml version=""1.0"" encoding=""UTF-8""?>")
        XmlLine.Add("<beginCertificate action=""revocate"">")
        XmlLine.Add("  <certificate>")                                 ' 証明書の内容を定義します
        XmlLine.Add("    <serialNumber>" & Me.SerialNumber.ToString() & "</serialNumber>")
        If Me.ReasonCode = 1 Then                                      ' -> 証明書の失効理由
            XmlLine.Add("    <reasonCode>" & CommonDefines.ReasonCode1 & "</reasonCode>")
        ElseIf Me.ReasonCode = 2 Then
            XmlLine.Add("    <reasonCode>" & CommonDefines.ReasonCode2 & "</reasonCode>")
        ElseIf Me.ReasonCode = 3 Then
            XmlLine.Add("    <reasonCode>" & CommonDefines.ReasonCode3 & "</reasonCode>")
        ElseIf Me.ReasonCode = 4 Then
            XmlLine.Add("    <reasonCode>" & CommonDefines.ReasonCode4 & "</reasonCode>")
        ElseIf Me.ReasonCode = 5 Then
            XmlLine.Add("    <reasonCode>" & CommonDefines.ReasonCode5 & "</reasonCode>")
        ElseIf Me.ReasonCode = 6 Then
            XmlLine.Add("    <reasonCode>" & CommonDefines.ReasonCode6 & "</reasonCode>")
        ElseIf Me.ReasonCode = 7 Then
            XmlLine.Add("    <reasonCode>" & CommonDefines.ReasonCode7 & "</reasonCode>")
        ElseIf Me.ReasonCode = 8 Then
            XmlLine.Add("    <reasonCode>" & CommonDefines.ReasonCode8 & "</reasonCode>")
        ElseIf Me.ReasonCode = 9 Then
            XmlLine.Add("    <reasonCode>" & CommonDefines.ReasonCode9 & "</reasonCode>")
        End If                                                         ' -> 証明書の失効日
        XmlLine.Add("    <revocateDate timezone=""JST"">" & Me.RevocateDate & "</revocateDate>")
        XmlLine.Add("  </certificate>")
        XmlLine.Add("</beginCertificate>")

        Try                                                            ' XMLを一時ファイルに書き込みます
            Using DataFile As StreamWriter = New StreamWriter(K9piaTemp, False)
                For i As Integer = 0 To XmlLine.Count - 1 Step 1
                    DataFile.WriteLine(XmlLine(i))
                Next
                DataFile.Close()
            End Using
            Debug.WriteLine("XMLファイルを作成しました。")
            Debug.WriteLine("[" & K9piaTemp & "]")
        Catch ex As Exception
            Debug.WriteLine("XMLファイルの作成に失敗しました。")
            Debug.WriteLine("[" & K9piaTemp & "]")
            Debug.WriteLine(ex.Message)
            RetCode = ErrorCodeK9piaXml
        End Try

        If RetCode = 0 Then
            Try                                                        ' 発行局へ証明書発行コマンドを送ります
                Dim ChildCode As String
                Dim TimeoutFlag As Boolean
                Dim TimeoutMillisec As Integer = 600 * 1000
                Dim CommandLine As String = String.Format("{0}{1}", Me.K9piaExecDirectory, Me.K9piaExecName)
                Dim Parame1Line As String = String.Format("""{0}""", K9piaTemp)
                Dim ExecK9pia As System.Diagnostics.Process            ' コンソール画面をワザと表示させています
                ExecK9pia = System.Diagnostics.Process.Start(CommandLine, Parame1Line)
                ' Dim ProcInfo As New ProcessStartInfo(CommandLine, Parame1Line)
                ' ProcInfo.CreateNoWindow = True
                ' ProcInfo.UseShellExecute = False
                ' ExecK9pia = System.Diagnostics.Process.Start(ProcInfo)
                TimeoutFlag = ExecK9pia.WaitForExit(TimeoutMillisec)
                If TimeoutFlag Then
                    ChildCode = ExecK9pia.ExitCode
                    If ChildCode = 0 Then                              ' 発行局が正常終了した場合
                        Debug.WriteLine("証明書を発行しました。")
                        RetCode = 0
                        System.IO.File.Delete(K9piaTemp)               ' 発行用XMLファイルを消去します
                    Else                                               ' 発行局がエラー終了した場合
                        Debug.WriteLine("証明書の発行に失敗しました。")
                        Debug.WriteLine("終了コード " & ChildCode.ToString())
                        RetCode = ChildCode
                        ' System.IO.File.Delete(K9piaTemp)             ' 発行用XMLファイルを消去します
                    End If
                Else                                                   ' 発行局がタイムアウトした場合
                    Debug.WriteLine("証明書の発行でタイムアウトが発生しました。")
                    RetCode = ErrorCodeK9piaTimeout
                    ' System.IO.File.Delete(K9piaTemp)                 ' 発行用XMLファイルを消去します
                End If
            Catch ex As Exception                                      ' その他の異常の場合
                Debug.WriteLine("証明書の発行に失敗しました。")
                Debug.WriteLine(ex.Message)
                RetCode = ErrorCodeK9piaCreate
                ' System.IO.File.Delete(K9piaTemp)                     ' 発行用XMLファイルの消去
            End Try
        End If
        Return RetCode
    End Function
End Class
