﻿' * 名前 : k9piaInitialSetup.vb
' * 機能 : 発行局の初期化
' * 作成 : 2014/04/-- Ver1.0
' * 更新 : ----/--/-- Ver -.-
' * 備考 : XMLインターフェースを使用して、発行局を初期化します。
' *        XMLファイルの詳細は「40 発行局XMLインターフェース仕様書.pdf」を参照してください。
' *

Imports Microsoft.VisualBasic.ControlChars
Imports System.IO
Imports System.Text
Imports System.Diagnostics

Public Class k9piaInitialSetup
    Public Property K9piaExecDirectory As String                       ' 発行局の実行ディレクトリを保持します
    Public Property K9piaExecName As String                            ' 発行局の実行ファイル名を保持します
    Public Property K9piaManageDirectory As String                     ' 発行局の管理情報ディレクトリを保持します
    Public Property K9piaManageName As String                          ' 発行局の管理情報ファイル名を保持します
    Public Property K9piaLogDirectory As String                        ' 発行局のログディレクトリを保持します
    Public Property K9piaLogName As String                             ' 発行局のログファイル名を保持します

    Public Sub New()                                                   ' コンストラクタ
        Me.K9piaExecDirectory = System.Windows.Forms.Application.StartupPath & "/bin/"
        Me.K9piaExecName = "k9pia.exe"                                 ' ここで一応の初期化はしていますが、
        'Me.K9piaManageDirectory = Me.K9piaExecDirectory & "files/"     ' 呼び出し元で再設定しています。
        Me.K9piaManageDirectory = "/files/"
        Me.K9piaManageName = "Serial.db"
        'Me.K9piaLogDirectory = Me.K9piaExecDirectory & "log/"
        Me.K9piaLogDirectory = "/log/"
        Me.K9piaLogName = "k9pia"
    End Sub

    Public Function sendCommand() As Integer                           ' 発行局の初期化を行います
        Dim RetCode As Integer = 0
        Dim CheckDir As DirectoryInfo = New DirectoryInfo(Me.K9piaExecDirectory & Me.K9piaManageDirectory)
        If Not CheckDir.Exists Then                                    ' ディレクトリが存在しない場合には作成します
            CheckDir.Create()
            Debug.WriteLine("ディレクトリ" & Me.K9piaExecDirectory & Me.K9piaManageDirectory & "を作成しました。")
        End If
        Dim CheckDir2 As DirectoryInfo = New DirectoryInfo(Me.K9piaExecDirectory & Me.K9piaLogDirectory)
        If Not CheckDir2.Exists Then
            CheckDir2.Create()
            Debug.WriteLine("ディレクトリ" & Me.K9piaExecDirectory & Me.K9piaLogDirectory & "を作成しました。")
        End If

        Dim XmlLine(13) As String                                      ' 初期化用のXMLファイルを作成します
        Dim K9piaTemp As String = Me.K9piaExecDirectory & "k9pia_InitialSetup.xml"
        XmlLine(0) = "<?xml version=""1.0"" encoding=""UTF-8""?>"
        XmlLine(1) = "<initialSetup>"
        XmlLine(2) = "  <defines>"
        XmlLine(3) = "    <manageFiles>"
        XmlLine(4) = "      <directory>" & Me.K9piaManageDirectory & "</directory>"
        XmlLine(5) = "      <filename>" & Me.K9piaManageName & "</filename>"
        XmlLine(6) = "    </manageFiles>"
        XmlLine(7) = "    <logFiles>"
        XmlLine(8) = "      <directory>" & Me.K9piaLogDirectory & "</directory>"
        XmlLine(9) = "      <filename>" & Me.K9piaLogName & "</filename>"
        XmlLine(10) = "      <rotation>none</rotation>"
        XmlLine(11) = "    </logFiles>"
        XmlLine(12) = "  </defines>"
        XmlLine(13) = "</initialSetup>"
        Try
            Using DataFile As StreamWriter = New StreamWriter(K9piaTemp, False)
                For i As Integer = 0 To XmlLine.Length - 1 Step 1
                    DataFile.WriteLine(XmlLine(i))
                Next
                DataFile.Close()
            End Using
            Debug.WriteLine("XMLファイルを作成しました。")
            Debug.WriteLine("[" & K9piaTemp & "]")
        Catch ex As Exception
            Debug.WriteLine("XMLファイルの作成に失敗しました。")
            Debug.WriteLine("[" & K9piaTemp & "]")
            Debug.WriteLine(ex.Message)
            RetCode = ErrorCodeK9piaXml
        End Try

        If RetCode = 0 Then
            Try                                                        ' 発行局へ初期化コマンドを送ります
                Dim ChildCode As String
                Dim TimeoutFlag As Boolean
                Dim TimeoutMillisec As Integer = 600 * 1000
                Dim CommandLine As String = String.Format("{0}{1}", Me.K9piaExecDirectory, Me.K9piaExecName)
                Debug.WriteLine("[" & CommandLine & "]")
                Dim Parame1Line As String = String.Format("""{0}""", K9piaTemp)
                Debug.WriteLine("[" & Parame1Line & "]")
                Dim ExecK9pia As System.Diagnostics.Process            ' コンソール画面を表示させずに実行します
                ' ExecK9pia = System.Diagnostics.Process.Start(CommandLine, Parame1Line)
                Dim ProcInfo As New ProcessStartInfo(CommandLine, Parame1Line)
                ProcInfo.CreateNoWindow = True
                ProcInfo.UseShellExecute = False
                ExecK9pia = System.Diagnostics.Process.Start(ProcInfo)
                TimeoutFlag = ExecK9pia.WaitForExit(TimeoutMillisec)
                If TimeoutFlag Then
                    ChildCode = ExecK9pia.ExitCode
                    If ChildCode = 0 Then                              ' 発行局が正常終了した場合
                        Debug.WriteLine("認証局を初期化しました。")
                        RetCode = 0
                        System.IO.File.Delete(K9piaTemp)               ' 初期化用XMLファイルを消去します
                    Else                                               ' 発行局がエラー終了した場合
                        Debug.WriteLine("認証局の初期化に失敗しました。")
                        Debug.WriteLine("終了コード " & ChildCode.ToString())
                        RetCode = ChildCode
                        ' System.IO.File.Delete(K9piaTemp)             ' 初期化用XMLファイルを消去します
                    End If
                Else                                                   ' 発行局がタイムアウトした場合
                    Debug.WriteLine("認証局の初期化でタイムアウトが発生しました。")
                    RetCode = ErrorCodeK9piaTimeout
                    ' System.IO.File.Delete(K9piaTemp)                 ' 初期化用XMLファイルを消去します
                End If
            Catch ex As Exception                                      ' その他の異常の場合
                Debug.WriteLine("認証局の初期化に失敗しました。")
                Debug.WriteLine(ex.Message)
                RetCode = ErrorCodeK9piaInit
                ' System.IO.File.Delete(K9piaTemp)                     ' 初期化用XMLファイルを消去します
            End Try
        End If
        Return RetCode
    End Function
End Class
