﻿' * 名前 : ASN1analyze.vb
' * 機能 : ASN.1のDERバイナリ形式を解析する
' * 作成 : 2014/--/-- Ver1.0
' * 更新 : ----/--/-- Ver -.-
' * 備考 : このクラスは再帰呼び出しを行います。
' * 　　   再帰呼び出しは、決して“使ってはいけない技術”などではありません。
' * 　　   正しく理解して使いこなせれば、何の問題もありません。
' * 　　   怖がらず、振り回されず、技術を習得しましょう。
' *

Imports Microsoft.VisualBasic.ControlChars
Imports System.IO
Imports System.Text
Imports System.Diagnostics

Public Class ASN1analyze
    Public Property MyType As Byte
    Public Property MySize As Int64
    Public Property MyData As Byte()
    Public Property ChildCount As Integer
    Public Property ChildElement As ASN1analyze()
    Public Property NextId As Int64
    Public Property TotalSize As Int64

    Const ASN1_VALIABLE As Byte = &H80
    Const ASN1_LENMASK As Byte = &H7F
    Const ASN1_BOL As Byte = &H1
    Const ASN1_INT As Byte = &H2
    Const ASN1_BIT As Byte = &H3
    Const ASN1_OCT As Byte = &H4
    Const ASN1_NUL As Byte = &H5
    Const ASN1_OID As Byte = &H6
    Const ASN1_ENM As Byte = &HA
    Const ASN1_UTF As Byte = &HC
    Const ASN1_PRT As Byte = &H13
    Const ASN1_TEL As Byte = &H14
    Const ASN1_VDO As Byte = &H15
    Const ASN1_IA5 As Byte = &H16
    Const ASN1_BMP As Byte = &H1E
    Const ASN1_UTC As Byte = &H17
    Const ASN1_SEQ As Byte = &H30
    Const ASN1_SET As Byte = &H31
    Const ASN1_CHO As Byte = &H80

    Public Sub New()
        Me.MyType = &H0
        Me.MySize = 0
        Me.MyData = Nothing
        Me.ChildCount = 0
        Me.ChildElement = Nothing
        Me.NextId = 0
        Me.TotalSize = 0
    End Sub

    Public Function RunAnalyze(ByVal BinTemp() As Byte) As Boolean
        Dim RetFlag As Boolean = False
        If BinTemp.Length > 1 Then
            If (BinTemp(0) = ASN1_BOL) Or (BinTemp(0) = ASN1_INT) Or (BinTemp(0) = ASN1_BIT) _
            Or (BinTemp(0) = ASN1_OCT) Or (BinTemp(0) = ASN1_NUL) Or (BinTemp(0) = ASN1_OID) _
            Or (BinTemp(0) = ASN1_ENM) Or (BinTemp(0) = ASN1_UTF) Or (BinTemp(0) = ASN1_PRT) _
            Or (BinTemp(0) = ASN1_TEL) Or (BinTemp(0) = ASN1_VDO) Or (BinTemp(0) = ASN1_IA5) _
            Or (BinTemp(0) = ASN1_BMP) Or (BinTemp(0) = ASN1_UTC) Or (BinTemp(0) = ASN1_SEQ) _
            Or (BinTemp(0) = ASN1_SET) Or (BinTemp(0) = ASN1_CHO) Or (BinTemp(0) = &H81) _
            Or (BinTemp(0) = &H82) Or (BinTemp(0) = &H83) Or (BinTemp(0) = &H84) _
            Or (BinTemp(0) = &H85) Or (BinTemp(0) = &H86) Or (BinTemp(0) = &HA0) _
            Or (BinTemp(0) = &HA1) Or (BinTemp(0) = &HA2) Or (BinTemp(0) = &HA3) _
            Or (BinTemp(0) = &HA4) Or (BinTemp(0) = &HA5) Or (BinTemp(0) = &HA6) Then
                Dim DataStart As Int64 = -1
                Dim DataLast As Int64 = -1
                Dim DataSize As Int64 = -1
                Dim AllSize As Int64 = -1
                If BinTemp(1) = &H0 Then
                    DataStart = 0
                    DataLast = 0
                    DataSize = 0
                    AllSize = 2
                ElseIf BinTemp(1) = ASN1_VALIABLE Then
                    For i As Int64 = 2 To BinTemp.Length - 2 Step 1
                        If (BinTemp(i) = 0) And (BinTemp(i + 1) = 0) Then
                            DataStart = 2
                            DataLast = i
                            DataSize = DataLast - DataStart
                            AllSize = 2 + DataSize
                            Exit For
                        End If
                    Next
                ElseIf (BinTemp(1) And ASN1_VALIABLE) = &H0 Then
                    Dim LenTemp As Integer = BinTemp(1) And ASN1_LENMASK
                    If (2 + LenTemp) <= BinTemp.Length Then
                        DataStart = 2
                        DataLast = DataStart + LenTemp
                        DataSize = LenTemp
                        AllSize = 2 + DataSize
                    End If
                Else
                    Dim LenTemp As Integer = BinTemp(1) And ASN1_LENMASK
                    If ((2 + LenTemp) <= BinTemp.Length) And (LenTemp < 4) Then
                        DataSize = 0
                        For i As Integer = 0 To LenTemp - 1 Step 1
                            DataSize = DataSize * 256 + BinTemp(2 + i)
                        Next
                        If (2 + LenTemp + DataSize) <= BinTemp.Length Then
                            DataStart = 2 + LenTemp
                            DataLast = DataStart + DataSize
                            AllSize = 2 + LenTemp + DataSize
                        End If
                    End If
                End If
                If AllSize > 0 Then
                    Me.MyType = BinTemp(0)
                    Me.MySize = DataSize
                    Me.NextId = DataLast
                    Me.TotalSize = AllSize
                    If DataSize > 0 Then
                        Me.MyData = New Byte() {}
                        Array.Resize(Me.MyData, DataSize)
                        System.Array.Copy(BinTemp, DataStart, Me.MyData, 0, DataSize)
                        If (Me.MyType = ASN1_OCT) Or (Me.MyType = ASN1_SEQ) Or (Me.MyType = ASN1_SET) Or (Me.MyType = ASN1_CHO) _
                        Or (Me.MyType = &H81) Or (Me.MyType = &H82) Or (Me.MyType = &H83) _
                        Or (Me.MyType = &H84) Or (Me.MyType = &H85) Or (Me.MyType = &H86) _
                        Or (Me.MyType = &HA0) Or (Me.MyType = &HA1) Or (Me.MyType = &HA2) _
                        Or (Me.MyType = &HA3) Or (Me.MyType = &HA4) Or (Me.MyType = &HA5) Or (Me.MyType = &HA6) Then
                            Me.ChildElement = New ASN1analyze() {}
                            Dim ChildInd As Int64 = 0
                            While ChildInd < Me.MySize
                                Dim RemindSize As Int64 = Me.MySize - ChildInd
                                Dim ChildTemp As Byte() = {}
                                Array.Resize(ChildTemp, RemindSize)
                                System.Array.Copy(Me.MyData, ChildInd, ChildTemp, 0, RemindSize)
                                Array.Resize(Me.ChildElement, (Me.ChildCount + 1))
                                Me.ChildElement(Me.ChildCount) = New ASN1analyze()
                                If Me.ChildElement(Me.ChildCount).RunAnalyze(ChildTemp) Then
                                    ChildInd += Me.ChildElement(Me.ChildCount).TotalSize
                                    Me.ChildCount += 1
                                Else
                                    For i As Integer = 0 To Me.ChildCount - 1 Step 1
                                        Me.ChildElement(i) = Nothing
                                    Next
                                    Me.ChildCount = 0
                                    ChildTemp = Nothing
                                    Exit While
                                End If
                                ChildTemp = Nothing
                            End While
                        ElseIf (Me.MyType = ASN1_BIT) And (Me.MyData(0) = &H0) Then
                            Me.ChildElement = New ASN1analyze() {}
                            Dim ChildInd As Int64 = 1
                            While ChildInd < Me.MySize
                                Dim RemindSize As Int64 = Me.MySize - ChildInd
                                Dim ChildTemp As Byte() = {}
                                Array.Resize(ChildTemp, RemindSize)
                                System.Array.Copy(Me.MyData, ChildInd, ChildTemp, 0, RemindSize)
                                Array.Resize(Me.ChildElement, (Me.ChildCount + 1))
                                Me.ChildElement(Me.ChildCount) = New ASN1analyze()
                                If Me.ChildElement(Me.ChildCount).RunAnalyze(ChildTemp) Then
                                    ChildInd += Me.ChildElement(Me.ChildCount).TotalSize
                                    Me.ChildCount += 1
                                Else
                                    For i As Integer = 0 To Me.ChildCount - 1 Step 1
                                        Me.ChildElement(i) = Nothing
                                    Next
                                    Me.ChildCount = 0
                                    ChildTemp = Nothing
                                    Exit While
                                End If
                                ChildTemp = Nothing
                            End While
                        End If
                    End If
                    RetFlag = True
                End If
            End If
        End If
        Return RetFlag
    End Function

    Public Function GetText() As String
        Dim RetText As String = ""
        Dim IsPrintable As Boolean = True
        For i As Int64 = 0 To Me.MyData.Length - 1 Step 1
            Dim TempUint As UInt16 = System.Convert.ToUInt16(Me.MyData(i))
            If TempUint > 127 Then
                IsPrintable = False
                Exit For
            End If
        Next
        If IsPrintable Then
            Try
                Dim Ascii As ASCIIEncoding = New ASCIIEncoding()
                RetText = Ascii.GetString(Me.MyData)
            Catch ex As Exception
                RetText = Nothing
            End Try
        Else
            Try
                Dim Utf8 As UTF8Encoding = New UTF8Encoding()
                RetText = Utf8.GetString(Me.MyData)
            Catch ex As Exception
                RetText = Nothing
            End Try
        End If
        Return RetText
    End Function

    Public Function GetOID() As String
        Dim RetText As String = ""
        If Me.MySize > 1 Then
            Dim Oid1st As Integer = System.Convert.ToUInt16(Me.MyData(0)) / 40
            Dim Oid2nd As Integer = System.Convert.ToUInt16(Me.MyData(0)) Mod 40
            RetText = Oid1st.ToString() & "." & Oid2nd.ToString()
            Dim IdTemp As Int64 = 0
            For i As Int64 = 1 To Me.MySize - 1 Step 1
                IdTemp = IdTemp * 128 + System.Convert.ToUInt16(Me.MyData(i) And &H7F)
                If (Me.MyData(i) And &H80) = 0 Then
                    RetText &= "." & IdTemp.ToString()
                    IdTemp = 0
                End If
            Next
        Else
            RetText = Nothing
        End If
        Return RetText
    End Function

    Public Function GetDate() As Date
        Dim RetDate As Date = New Date(1970, 1, 1, 0, 0, 0)
        Try
            Dim Ascii As ASCIIEncoding = New ASCIIEncoding()
            Dim TempText As String = Ascii.GetString(Me.MyData)
            Dim TempUtcs As String
            If TempText.Length = 13 Then
                TempUtcs = String.Format("20{0}/{1}/{2} {3}:{4}:{5}", TempText.Substring(0, 2), TempText.Substring(2, 2) _
                , TempText.Substring(4, 2), TempText.Substring(6, 2), TempText.Substring(8, 2), TempText.Substring(10, 2))
            Else
                TempUtcs = String.Format("{0}/{1}/{2} {3}:{4}:{5}", TempText.Substring(0, 4), TempText.Substring(4, 2) _
                , TempText.Substring(6, 2), TempText.Substring(8, 2), TempText.Substring(10, 2), TempText.Substring(12, 2))
            End If
            RetDate = System.Convert.ToDateTime(TempUtcs)
        Catch ex As Exception
            RetDate = Nothing
        End Try
        Return RetDate
    End Function

    Public Function GetInt64() As Int64
        Dim RetValue As Int64 = 0
        If Me.MySize > 0 Then
            For i As Int64 = 0 To Me.MySize - 1 Step 1
                RetValue = RetValue * 256 + System.Convert.ToUInt16(Me.MyData(i))
            Next
        Else
            RetValue = -1
        End If
        Return RetValue
    End Function
End Class
